<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('BOOKZO_PLATFORM') or die;

/**
 * Bz framework class.
 *
 * This class acts as the generic dispatcher and provides
 * also access to every helper available through the use
 * of the magic method __get
 *
 * @subpackage 	framework.classes
 * @since 		1.0.0
 */
class Bookzo {
	
	/**
	 * The framework version
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
    const VERSION = '1.2.13';
	
	/**
	 * Unique identifier for this App
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
    public $id;
	
	/**
	 * The classname
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
    public $path;
	
	/**
	 * The classname
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
    protected $_name;
	
	/**
	 * The list of helpers currently loaded
	 *
	 * @var 	array
	 * @since 	1.0.0
	 */
	protected $_helpers = array();
	
	/**
	 * Static container for the applications instances
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected static $instance = null;
	
	protected $_defaults = false;
	
	/**
	 * Class constructor
	 *
	 * @param 	string $id The unique identifier of the application
	 * @since 	1.0.0
	 */
	protected function __construct() 
	{
		// init vars
		$this->_name = strtolower(get_class($this));

		// set defaults
		$path = dirname(__DIR__);
		
		// add pathhelper
		$this->path = BPathHelper::getInstance('bookzo');
		
		// register default paths
		$this->path->register(BOOKZO_ROOT.'/controllers', 	'controllers');
		$this->path->register(BOOKZO_ROOT.'/models', 		'models');
		$this->path->register(BOOKZO_ROOT.'/views', 		'views');
		$this->path->register(BOOKZO_ROOT.'/widgets', 		'widgets');
		$this->path->register(BOOKZO_ROOT.'/plugins', 		'plugins');
		$this->path->register(BOOKZO_ROOT.'/adapters', 		'adapters');
		$this->path->register(BOOKZO_ROOT.'/templates', 	'templates');
		$this->path->register(BOOKZO_ROOT.'/language', 		'languages');
		$this->path->register(BOOKZO_ROOT.'/assets', 		'assets');
		
	}
	
	/**
	 * Gets an instance of an application
	 *
	 * @param 	string $id The application unique identifier
	 * @return 	App A Bz Object
	 * @since 	1.0.0
	 */
	public static function getInstance() 
	{		
		// add instance, if not exists
		if (!isset(self::$instance)) 
		{
			self::$instance 	= new Bookzo();
		}

		return self::$instance;
	}
	
	public function loadDefaults()
	{
		if ( $this->_defaults == false ) 
		{
	 		$this->_defaults = true;
			
			// load adapter override paths first
			$this->adapter->addPaths();
			
			// load the language (dutch by default)
			$this->language->load('root', 'nl');
			$this->language->load('root', null, true);
			
			// merge settings
			if ( $settings = $this->api->getSettings() ) {
				$this->config->add( (object) $settings, 'bookzo');
			}
			
			// add default css files if not in ajax mode
			if ( $this->input->get('format') != 'ajax' ) 
			{
				$this->adapter->addScriptDeclaration('window["BOOKZO_URL"]  = "'. BOOKZO_URL . '";');
				$this->adapter->addScriptDeclaration('window["BOOKZO_LANGID"]  = "'. $this->language->getId() . '";');
						
				$this->adapter->addStyleSheet('assets:css/grid.css');	
				$this->adapter->addStyleSheet('assets:css/fontawesome.css');	
				$this->adapter->addStyleSheet('assets:css/style.css');
				$this->adapter->addStyleSheet('assets:css/custom.css');
				
				$this->adapter->jQuery(true);
				$this->adapter->addScript('assets:js/grid.js');
				$this->adapter->addScript('assets:js/bookzo.js');
			}
			
			// load plugins
			$this->plugin->load();
			
			// load first event
			$this->event->dispatcher->fire(	$this->event->create('bookzo:init') );
			$this->adapter->bookzoInit();
			
			/*
			## NOTE - Look into this. We want to clear data if we are on a normal CMS page
			if ( !$this->adapter->isBookzoPage() ) {
				// clear some bookzo sessions
				$this->adapter->sessionClear('bookzo.pagetype');
				$this->adapter->sessionClear('bookzo.search.options');
				$this->adapter->sessionClear('bookzo.search.results');
				$this->adapter->sessionClear('bookzo.search.query');
			}
			*/
		}
	}
	
	/**
	 * Get the name of app
	 *
	 * @return 	string The name of the helper
	 * @param 	boolean $first If true return first character capitalized
	 * @since 	1.0.0
	 */
	public function getName($first = true) 
	{
		if ( $first ) 
		{
			return ucfirst($this->_name);
		}
		
		return $this->_name;
	}
	
	/**
	 * Retrives an helper file
	 *
	 * @param 	string $name The name of the helper to retrieve
	 * @return 	AppHelper The helper instance requested
	 * @since 	1.0.0
	 */
	public function getHelper($name) 
	{
		// try to load helper, if not found
		if (!isset($this->_helpers[$name])) 
		{
		    $this->loadHelper($name);
		}

		// get helper
		if (isset($this->_helpers[$name])) 
		{
			return $this->_helpers[$name];
		}

		return null;
	}
	
	/**
	 * Adds a helper
     *
	 * @param 	AppHelper $helper Helper object
	 * @param 	string $alias Optional Helper alias
	 * @since 	1.0.0
	*/
	public function addHelper($helper, $alias = null) 
	{
		// add to helpers
		$name = $helper->getName();
		$this->_helpers[$name] = $helper;

		// add alias
		if (!empty($alias)) 
		{
			$this->_helpers[$alias] = $helper;
		}

	}
	
	/**
	 * Load an helper from the path
	 *
	 * @param string|array $helpers	Helper name or list of helper names to load
	 * @param string $suffix The suffix of the helper class. Default is "Helper"
	 * @since 1.0.0
	 */
	public function loadHelper($helpers) {
		
		$helpers = (array) $helpers;

		foreach ($helpers as $name) 
		{
			$class = 'BzHelper'.$name;

			// autoload helper class
			if (!class_exists($class))
			{
				if ( $file = $this->path->path('helpers:'.$name.'.php')) { 
				   	require_once($file);
				}
				else {
					throw new BzException('Helper '. $name . ' not found');
				}
			}

			// add helper, if not exists
			if (!isset($this->_helpers[$name])) 
			{
				$this->addHelper(new $class($this));
			}
		}
	}
	
	/**
	 * Retrieve an Helper
	 *
	 * @param 	string $name The helper name
	 * @return 	AppHelper The helper instance requested
	 * @since 	1.0.0
	 */
	public function __get($name) 
	{
		return $this->getHelper($name);
	}
	
	public function render()
	{		
		$controller = BController::getInstance($this);
		$controller->execute($this->input->get('task', null));
	}
	
	public function ajaxRender()
	{
		$this->input->set('format', 'ajax');
		
		$controller = BController::getInstance($this);
		return $controller->execute($this->input->get('task', null));
	}
	
	## NOTE - Shouldn't this be in separate class like state or something
	/**
	 * Gets a user state.
	 *
	 * @param   string  $key      The path of the state.
	 * @param   mixed   $default  Optional default value, returned if the internal value is null.
	 *
	 * @return  mixed  The user state or null.
	 *
	 * @since   3.2
	 */
	public function getUserState($key, $default = null)
	{
		$userstate = $this->adapter->sessionGet('bookzo.userstate');
		$registry  = new BRegistry($userstate);
		
		return $registry->get($key, $default);
	}
	
	## NOTE - Shouldn't this be in separate class like state or something
	/**
	 * Sets the value of a user state variable.
	 *
	 * @param   string  $key    The path of the state.
	 * @param   string  $value  The value of the variable.
	 *
	 * @return  mixed  The previous state, if one existed.
	 *
	 * @since   3.2
	 */
	public function setUserState($key, $value)
	{
		$userstate = $this->adapter->sessionGet('bookzo.userstate');
		$registry  = new BRegistry($userstate);

		$return = $registry->set($key, $value);
		$this->adapter->sessionSet('bookzo.userstate', $registry->toString());
			
		return $return;
	}
	
	## NOTE - Shouldn't this be in separate class like state or something
	/**
	 * Gets the value of a user state variable.
	 *
	 * @param   string  $key      The key of the user state variable.
	 * @param   string  $request  The name of the variable passed in a request.
	 * @param   string  $default  The default value for the variable if not found. Optional.
	 * @param   string  $type     Filter for the variable, for valid values see {@link JFilterInput::clean()}. Optional.
	 *
	 * @return  object  The request user state.
	 *
	 * @since   3.2
	 */
	public function getUserStateFromRequest($key, $request, $default = null, $type = 'none')
	{
		$cur_state = $this->getUserState($key, $default);
		$new_state = $this->input->get($request, null, $type);

		// Save the new value only if it was set in this request.
		if ($new_state !== null)
		{
			$this->setUserState($key, $new_state);
		}
		else
		{
			$new_state = $cur_state;
		}

		return $new_state;
	}
}

/**
 * BzException identifies an Exception in the Bz class
 * @see Bz
 */
class BzException extends Exception {

	/**
	 * Converts the exception to a human readable string
	 *
	 * @return 	string The error message
	 * @since 	1.0.0
	 */
	public function __toString() 
	{
		return $this->getMessage();
	}

}