<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

/**
 * Bz helper class.
 *
 * This class acts as parent for all helper classes
 * It adds default functions like a magic method call 
 * for all child classes
 *
 * @subpackage 	framework.classes
 * @since 		1.0.0
 */
class BController {

	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	public $app;
	
	/**
	 * Hold a BInput object for easier access to the input variables.
	 *
	 * @var    BInput
	 * @since  12.2
	 */
	public $input;
	
	/**
	 * Instance container.
	 *
	 * @var    JControllerLegacy
	 * @since  12.2
	 */
	protected static $instance;
	
	/**
	 * Instance container containing the views.
	 *
	 * @var    array
	 * @since  3.4
	 */
	protected static $views;

	/**
	 * Array of class methods
	 *
	 * @var    array
	 * @since  12.2
	 */
	protected $_methods;

	/**
	 * The name of the controller
	 *
	 * @var    array
	 * @since  12.2
	 */
	protected $_name;
	
	/**
	 * URL for redirection.
	 *
	 * @var    string
	 * @since  12.2
	 */
	protected $_redirect;

	/**
	 * Current or most recently performed task.
	 *
	 * @var    string
	 * @since  12.2
	 */
	protected $_task;

	/**
	 * Array of class methods to call for a given task.
	 *
	 * @var    array
	 * @since  12.2
	 */
	protected $_taskMap;
	
	
	/**
	 * The default task to execute in this controller
	 *
	 * @var    array
	 * @since  12.2
	 */
	protected $_defaultTask = 'display';
	
	/**
	 * The default task to execute in this controller
	 *
	 * @var    array
	 * @since  12.2
	 */
	protected $_defaultView;
	
	/**
	 * Redirect message.
	 *
	 * @var    string
	 * @since  12.2
	 */
	protected $_message;

	/**
	 * Redirect message type.
	 *
	 * @var    string
	 * @since  12.2
	 */
	protected $_messageType;
	



	/**
	 * Class constructor
	 *
	 * @param App $app A reference to an App Object
	 */
	public function __construct($name) 
	{
		// set application
		$this->app 	 		= Bookzo::getInstance();
		$this->input		= $this->app->input;
		
		$this->_methods 	= array();
		$this->_message 	= null;
		$this->_messageType = 'message';
		$this->_redirect 	= null;
		$this->_taskMap 	= array();
		$this->_name		= $name;
		
		// Determine the callable methods to remove from (child) class
		$xMethods 	= get_class_methods('BController');

		// Get the public methods in (child) Class.
		$r 			= new ReflectionClass($this);
		$rMethods 	= $r->getMethods(ReflectionMethod::IS_PUBLIC);

		// Only add methods that are not in base class 
		foreach ($rMethods as $rMethod)
		{
			$mName = $rMethod->getName();

			if (!in_array($mName, $xMethods) || $mName == 'display')
			{
				// add to method list and callable tasks
				$this->methods[] = strtolower($mName);
				$this->taskMap[strtolower($mName)] = $mName;
			}
		}

		// Set the view name
		if (empty($this->_name))
		{
			$this->_name = 'group';
		}

		// Set the default view.
		$this->_defaultView = $this->getName();		
	}
	
	/**
	 * Method to get a singleton controller instance.
	 *
	 * @param   string  $prefix  The prefix for the controller.
	 * @param   array   $config  An array of optional constructor options.
	 *
	 * @return  JControllerLegacy
	 *
	 * @since   12.2
	 * @throws  Exception if the controller cannot be loaded.
	 */
	public static function getInstance()
	{
		// if defined no need to continue
		if (is_object(self::$instance))
		{
			return self::$instance;
		}
		
		## NOTE - what layout to set as default

		// get correct layout and task
		$app		= Bookzo::getInstance();
		$format   	= $app->input->getWord('format');
		$type		= $app->input->getWord('layout', 'group');
		$command	= $app->input->get('task', 'display');

		// Check for a controller.task command.
		if (strpos($command, '.') !== false)
		{
			// Explode the controller.task command.
			list ($type, $task) = explode('.', $command);
			
			// Reset the task without the controller context.
			$app->input->set('layout', $type);
			$app->input->set('task', $task);
		}
		
		// Define the controller filename
		$files	 = self::createFileNames('controller', array('name' => $type, 'format' => $format));

		// Get the controller class name.
		$class = 'BzController' . ucfirst($type);

		// Register the class
		foreach ( $files as $file ) 
		{
			if ( $path = $app->path->path('controllers:'.$file) ) 
			{
				$app->loader->register($class, 'controllers:'.$file);
				break;
			}
		}

		## NOTE - for now we take base class if file nor present
		// we might want to reconsider or create a controller with correct naming through reflection
		
		// Get correct controller
		self::$instance = $app->loader->get(array($class, 'BControllerDefault'), $type);

		// return
		return self::$instance;
	}
	
	/**
	 * Typical view method for MVC based architecture
	 *
	 * This function is provide as a default implementation, in most cases
	 * you will need to override it in your own controllers.
	 *
	 * @param   boolean  $cachable   If true, the view output will be cached
	 * @param   array    $urlparams  An array of safe url parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
	 *
	 * @return  JControllerLegacy  A JControllerLegacy object to support chaining.
	 *
	 * @since   12.2
	 */
	public function display()
	{
		//$document = JFactory::getDocument();
		//$viewType = $document->getType();
		$viewType	= '';
		$viewName 	= $this->input->get('layout', $this->_defaultView);
		$view 		= $this->getView($viewName, $viewType);

		// Get/Create the model
		if ($model = $this->getModel($viewName))
		{
			// Push the model into the view (as default)
			$view->setModel($model, true);
		}

		$view->display();

		return $this;
	}
	
	/**
	 * Get a reference to the current view and load it if necessary
	 *
	 * @param string $name The name of the view to load. By default load the current view
	 * @param string $type The view type. Optional.
	 * @param string $prefix The class prefix. Optional.
	 * @param array $config An optional configuration array for the view
	 *
	 * @return AppView The view class requested
	 *
	 * @since 1.0.0
	 */
	public function getView($name = '', $type = '', $config = array()) {

		if (!isset(self::$views))
		{
			self::$views = array();
		}
		 
		// set name
		if (empty($name)) {
			$name = $this->getName();
		}
		
		// get class and file
		$class	  = 'BzView'.ucfirst($name);
		$files	  = self::createFileNames('view', array('name' => $name, 'type' => $type));
		
		// don't we allready know this view
		if (empty(self::$views[$name][$type]))
		{
			foreach ( $files as $file ) 
			{
				if ( $path = $this->app->path->path('views:'.$file) ) 
				{
					$this->app->loader->register($class, 'views:'.$file);
					break;
				}
			}
			
			// Get correct controller
			if ( $view = $this->app->loader->get($class, $config) )
			{
				self::$views[$name][$type] = $view;
			}
			else {
				
				## NOTE - MAYBE RAISE 404 ERROR
				
				//throw new BControllerException(sprintf('JLIB_APPLICATION_ERROR_VIEW_NOT_FOUND', $name, $type), 500);
				throw new BControllerException('JLIB_APPLICATION_ERROR_VIEW_NOT_FOUND'. $name. $type, 500);
			}

		}

		return self::$views[$name][$type];
	}
	
	/**
	 * Method to get a model object, loading it if required.
	 *
	 * @param   string  $name    The model name. Optional.
	 * @param   string  $prefix  The class prefix. Optional.
	 * @param   array   $config  Configuration array for model. Optional.
	 *
	 * @return  object  The model.
	 *
	 * @since   12.2
	 */
	public function getModel($name = '', $config = array())
	{
		if (empty($name))
		{
			$name = $this->getName();
		}
		
		// dont cache internally.
		// models can be called throughout the app
		
		## NOTE - we might want to do this like $this->app->model->name;
		
		// should always work since model takes base class if not present
		$model = BModel::getInstance($name, $config);
		$model->setState('task', $this->task);

		return $model;
	}
	


	
	/**
	 * Execute a task by triggering a method in the derived class.
	 *
	 * @param   string  $task  The task to perform. If no matching task is found, the '__default' task is executed, if defined.
	 *
	 * @return  mixed   The value returned by the called method, false in error case.
	 *
	 * @since   12.2
	 * @throws  Exception
	 */
	public function execute($task)
	{
		// add to global
		$this->task = $task;

		$task 		= strtolower($task);
		$doTask		= $this->_defaultTask;

		// find correct task
		if (isset($this->taskMap[$task]))
		{
			$doTask = $this->taskMap[$task];
		}

		// Execute the task
		$this->doTask = $doTask;

		return $this->$doTask();
	}
	
	/**
	 * Register (map) a task to a method in the class.
	 *
	 * @param   string  $task    The task.
	 * @param   string  $method  The name of the method in the derived class to perform for this task.
	 *
	 * @return  JControllerLegacy  A JControllerLegacy object to support chaining.
	 *
	 * @since   12.2
	 */
	public function registerTask($task, $method)
	{
		if (in_array(strtolower($method), $this->methods))
		{
			$this->taskMap[strtolower($task)] = $method;
		}

		return $this;
	}
	
	/**
	 * Unregister (unmap) a task in the class.
	 *
	 * @param   string  $task  The task.
	 *
	 * @return  JControllerLegacy  This object to support chaining.
	 *
	 * @since   12.2
	 */
	public function unregisterTask($task)
	{
		unset($this->taskMap[strtolower($task)]);

		return $this;
	}
	
	/**
	 * Get the last task that is being performed or was most recently performed.
	 *
	 * @return  string  The task that is being performed or was most recently performed.
	 *
	 * @since   12.2
	 */
	public function getTask()
	{
		return $this->task;
	}

	/**
	 * Gets the available tasks in the controller.
	 *
	 * @return  array  Array[i] of task names.
	 *
	 * @since   12.2
	 */
	public function getTasks()
	{
		return $this->methods;
	}
	
	/**
	 * Method to get the controller name
	 *
	 * The dispatcher name is set by default parsed using the classname, or it can be set
	 * by passing a $config['name'] in the class constructor
	 *
	 * @return  string  The name of the dispatcher
	 *
	 * @since   12.2
	 * @throws  Exception
	 */
	public function getName()
	{
		if (empty($this->_name))
		{
			$classname 		= get_class($this);
			$controllerpos 	= strpos($classname, 'Controller');

			if ($controllerpos === false)
			{
				throw new Exception('JLIB_APPLICATION_ERROR_VIEW_GET_NAME', 500);
			}

			$name = strtolower(substr($classname, $controllerpos + 10));
			$this->_name = empty($name) ? 'default' : $name;
		}

		return $this->_name;
	}

	
	/**
	 * Create the filename for a resource.
	 *
	 * @param   string  $type   The resource type to create the filename for.
	 * @param   array   $parts  An associative array of filename information. Optional.
	 *
	 * @return  string  The filename.
	 *
	 * @note    Replaced _createFileName.
	 * @since   12.2
	 */
	protected static function createFileNames($type, $parts = array())
	{
		$return = array();
		
		if (empty($parts['format']) || $parts['format'] == 'html' ) {
			$parts['format'] = '';
		}
		else {
			$parts['format'] = '.' . $parts['format'];
		}

		switch ( $type ) {
			case 'view' :
				$return[] = strtolower($parts['name'] . '/view' . $parts['format'] . '.php');
				$return[] = strtolower($parts['name'] . '/view.php');
				break;
			default:
			case 'controller':
				$return[] = strtolower($parts['name'] . $parts['format'] . '.php');
				$return[] = strtolower($parts['name'] . '.php');
				break;
		}
		
		return $return;
	}

	/**
	 * Redirects the browser or returns false if no redirect is set.
	 *
	 * @return  boolean  False if no redirect exists.
	 *
	 * @since   12.2
	 */
	public function redirect()
	{
		if ($this->redirect)
		{
			$app = JFactory::getApplication();

			// Enqueue the redirect message
			$app->enqueueMessage($this->message, $this->messageType);

			// Execute the redirect
			$app->redirect($this->redirect);
		}

		return false;
	}
	
	/**
	 * Set a URL for browser redirection.
	 *
	 * @param   string  $url   URL to redirect to.
	 * @param   string  $msg   Message to display on redirect. Optional, defaults to value set internally by controller, if any.
	 * @param   string  $type  Message type. Optional, defaults to 'message' or the type set by a previous call to setMessage.
	 *
	 * @return  JControllerLegacy  This object to support chaining.
	 *
	 * @since   12.2
	 */
	public function setRedirect($url, $msg = null, $type = null)
	{
		$this->redirect = $url;

		if ($msg !== null)
		{
			// Controller may have set this directly
			$this->message = $msg;
		}

		// Ensure the type is not overwritten by a previous call to setMessage.
		if (empty($type))
		{
			if (empty($this->messageType))
			{
				$this->messageType = 'message';
			}
		}
		// If the type is explicitly set, set it.
		else
		{
			$this->messageType = $type;
		}

		return $this;
	}
}

/**
 * The exception class dedicated for the controller classes
 *
 * @see AppController
 */
class BControllerException extends BzException {}