<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz Event class.
 *
 * This class acts as parent for all data classes
 * It adds default functions like a magic method call 
 * for all child classes
 *
 * @subpackage 	framework.classes
 * @since 		1.2.1
 */
class BEventDispatcher {

	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	public $app;
	
	/**
	 * Holder for registered events
	 *
	 * @var 	listeners
	 * @since 	1.2.1
	 */
	public $listeners;
	

	/**
	 * Class constructor
	 *
	 * @param App $app A reference to an App Object
	 */
	public function __construct() 
	{		
		// set application
		$this->app 	 	= Bookzo::getInstance();
	}
	
	/**
	 * Connects a listener to a given event name.
	 *
	 * @param string $name An event name
	 * @param mixed $listener A PHP callable
	 * 
	 * @since 1.0.0
	 */
	public function connect($name, $listener) {

		if (!isset($this->listeners[$name])) {
			$this->listeners[$name] = array();
		}

		$this->listeners[$name][] = $listener;
	}
	
	/**
	 * Disconnects a listener for a given event name.
	 *
	 * @param string $name An event name
	 * @param mixed $listener A PHP callable
	 *
	 * @return mixed false if listener does not exist, null otherwise
	 * 
	 * @since 1.0.0
	 */
	public function disconnect($name, $listener) {

		if (!isset($this->listeners[$name])) {
			return false;
		}

		foreach ($this->listeners[$name] as $i => $callable) {
			if ($listener === $callable) {
				unset($this->listeners[$name][$i]);
			}
		}
	}
	
	/**
	 * Returns true if the given event name has some listeners.
	 *
	 * @param string $name The event name
	 *
	 * @return boolean true if some listeners are connected, false otherwise
	 * 
	 * @since 1.0.0
	 */
	public function hasListeners($name) {

		if (!isset($this->listeners[$name])) {
			$this->listeners[$name] = array();
		}

		return (boolean) count($this->listeners[$name]);
	}

	/**
	 * Returns all listeners associated with a given event name.
	 *
	 * @param string $name The event name
	 *
	 * @return array An array of listeners
	 * 
	 * @since 1.0.0
	 */
	public function getListeners($name) {

		if (!isset($this->listeners[$name])) {
			return array();
		}

		return $this->listeners[$name];
	}
	
	/**
	 * Notifies all listeners of a given event.
	 *
	 * @param AppEvent $event A AppEvent instance
	 *
	 * @return AppEvent The AppEvent instance
	 * 
	 * @since 1.0.0
	 */
	public function fire( BEvent $event ) {
		foreach ($this->getListeners($event->getName()) as $listener) {
			call_user_func($listener, $event);
		}

		return $event;
	}

	/**
	 * Notifies all listeners of a given event until one returns a non null value.
	 *
	 * @param AppEvent $event A AppEvent instance
	 *
	 * @return AppEvent The AppEvent instance
	 * 
	 * @since 1.0.0
	 */
	 /*
	public function notifyUntil(AppEvent $event) {

		foreach ($this->getListeners($event->getName()) as $listener) {
			if (call_user_func($listener, $event)) {
				$event->setProcessed(true);
				break;
			}
		}

		return $event;
	}
	*/
}




/**
 * BzException identifies an Exception in the Bz class
 * @see Bz
 */
class BEventDispatcherException extends BRuntimeException {}