<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

class BEvent {

	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	public $app;
	
	/**
	 * The return value of the event
	 * 
	 * @var mixed
	 * @since 1.2.1
	 */
	protected $value = null;
	
	/**
	 * If the event has been processed by a listener
	 * 
	 * @var boolean
	 * @since 1.2.1
	 */
	protected $processed = false;
	
	/**
	 * The event subject
	 * 
	 * @var array/mixed
	 * @since 1.2.1
	 */
	protected $subjects = array();
	
	/**
	 * The event name
	 * 
	 * @var string
	 * @since 1.2.1
	 */
	protected $name = '';
	
	/**
	 * The parameters for the event
	 * 
	 * @var BRegistry Object
	 * @since 1.2.1
	 */
	public $params = null;

	/**
	 * Constructs a new BEvent.
	 *
	 * @param string 	$name 	The event name
	 * @param array 	$params An array of parameters
	 */
	public function __construct($name, $params = array()) {
		$this->app		= Bookzo::getInstance();
		$this->name 	= $name;
		$this->params	= $this->app->registry->create($params);
	}

	/**
	 * Add a subject to the event
	 *
	 * @return mixed The subject
	 * 
	 * @since 1.2.1
	 */
	public function addSubject($name, $value) {
		
		if ( !isset($this->subjects[$name]) ) {
			$this->subjects[$name] = $value;
		}
		return $this;
	}
	
	/**
	 * Add a subject to the event
	 *
	 * @return mixed The subject
	 * 
	 * @since 1.2.1
	 */
	public function setSubject($name, $value) {
		
		if ( !isset($this->subjects[$name]) ) {
			return $this;
		}
		return $this->subjects[$name] = $value;
	}
	
	/**
	 * Add multiple subjects to the event
	 *
	 * @return mixed The subject
	 * 
	 * @since 1.2.1
	 */
	public function addSubjects( Array $subjects ) {
		foreach ( $subjects as $k => $v ) {
			$this->addSubject($k, $v);
		}
		return $this;
	}
	
	/**
	 * Add multiple subjects to the event
	 *
	 * @return mixed The subject
	 * 
	 * @since 1.2.1
	 */
	public function getSubject( $name, $default = null ) {
		if ( isset($this->subjects[$name]) ) {
			return $this->subjects[$name];
		}
		return $default;
	}

	/**
	 * Returns the event name.
	 *
	 * @return string The event name
	 * 
	 * @since 1.2.1
	 */
	public function getName() {
		return $this->name;
	}
	
	/**
	 * Sets the return value for this event.
	 *
	 * @param mixed $value The return value
	 * @return AppEvent Return the object itself to allow chaining
	 * 
	 * @since 1.2.1
	 */
	public function setReturnValue($value) {
		$this->value = $value;
		return $this;
	}

	/**
	 * Returns the return value.
	 *
	 * @return mixed The return value
	 * @since 1.2.1
	 */
	public function getReturnValue() {
		return $this->value;
	}

	/**
	 * Sets the processed flag.
	 *
	 * @param boolean $processed The processed flag value
	 * @return AppEvent Return the object itself to allow chaining
	 * @since 1.2.1
	 */
	public function setProcessed($processed) {
		$this->processed = (boolean) $processed;
		return $this;
	}

	/**
	 * Returns whether the event has been processed by a listener or not.
	 *
	 * @return boolean true if the event has been processed, false otherwise 
	 * @since 1.2.1
	 */
	public function isProcessed() {
		return $this->processed;
	}
}

/**
 * BzException identifies an Exception in the Bz class
 * @see Bz
 */
class BEventException extends BRuntimeException {}
