<?php
/**
 * @package     Bookzo.Platform
 * @subpackage  FileSystem
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * A File handling class
 *
 * @since  11.1
 */
class BFile
{
	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	public $app;
	
	/**
	 * Class constructor
	 *
	 * @param App $app A reference to an App Object
	 */
	public function __construct($app) 
	{
		// set application
		$this->app 	 	= $app;
	}
	
	/**
	 * Gets the extension of a file name
	 *
	 * @param   string  $file  The file name
	 * @return  string  The file extension
	 * @since   1.0.0
	 */
	public function getExt($file)
	{
		$dot = strrpos($file, '.') + 1;

		return substr($file, $dot);
	}

	/**
	 * Strips the last extension off of a file name
	 *
	 * @param   string  $file  The file name
	 * @return  string  The file name without the extension
	 * @since   1.0.0
	 */
	public function stripExt($file)
	{
		return preg_replace('#\.[^.]*$#', '', $file);
	}

	/**
	 * Makes file name safe to use
	 *
	 * @param   string  $file  The name of the file [not full path]
	 * @return  string  The sanitised string
	 * @since   1.0.0
	 */
	public function makeSafe($file)
	{
		// Remove any trailing dots, as those aren't ever valid file names.
		$file = rtrim($file, '.');

		$regex = array('#(\.){2,}#', '#[^A-Za-z0-9\.\_\- ]#', '#^\.#');

		return trim(preg_replace($regex, '', $file));
	}

	/**
	 * Delete a file or array of files
	 *
	 * @param   mixed  $file  The file name or an array of file names
	 * @return  boolean  True on success
	 * @since   1.0.0
	 */
	public function delete($file)
	{
		// make sure we do more at once
		if (is_array($file)) {
			$files = $file;
		}
		else {
			$files[] = $file;
		}

		// loop all
		foreach ($files as $file)
		{
			$file = $this->app->filesystem->path->clean($file);

			if (!is_file($file))
			{
				continue;
			}

			// Try making the file writable first. If it's read-only, it can't be deleted
			// on Windows, even if the parent folder is writable
			@chmod($file, 0777);

			// In case of restricted permissions we zap it one way or the other
			// as long as the owner is either the webserver or the ftp
			if (@unlink($file))
			{
				// Do nothing
			}
			else
			{
				// $filename = basename($file);
				// BLog::add(BText::sprintf('JLIB_FILESYSTEM_DELETE_FAILED', $filename), BLog::WARNING, 'jerror');

				return false;
			}
		}

		return true;
	}

	/**
	 * Read the contents of a file
	 *
	 * @param   string   $filename   The full file path
	 * @param   boolean  $incpath    Use include path
	 * @param   integer  $amount     Amount of file to read
	 * @param   integer  $chunksize  Size of chunks to read
	 * @param   integer  $offset     Offset of the file
	 *
	 * @return  mixed  Returns file contents or boolean False if failed
	 * @since   1.0.0
	 */
	public static function read($filename, $incpath = false, $amount = 0, $chunksize = 8192, $offset = 0)
	{
		$data = null;

		if ($amount && $chunksize > $amount)
		{
			$chunksize = $amount;
		}

		if (false === $fh = fopen($filename, 'rb', $incpath))
		{
			return false;
		}

		clearstatcache();

		if ($offset)
		{
			fseek($fh, $offset);
		}

		if ($fsize = @ filesize($filename))
		{
			if ($amount && $fsize > $amount)
			{
				$data = fread($fh, $amount);
			}
			else
			{
				$data = fread($fh, $fsize);
			}
		}
		else
		{
			$data = '';

			/*
			 * While it's:
			 * 1: Not the end of the file AND
			 * 2a: No Max Amount set OR
			 * 2b: The length of the data is less than the max amount we want
			 */
			while (!feof($fh) && (!$amount || strlen($data) < $amount))
			{
				$data .= fread($fh, $chunksize);
			}
		}

		fclose($fh);

		return $data;
	}

	/**
	 * Write contents to a file
	 *
	 * @param   string   $file         The full file path
	 * @param   string   &$buffer      The buffer to write
	 * @return  boolean  True on success
	 * @since   1.0.0
	 */
	public function write($file, $buffer)
	{
		@set_time_limit(ini_get('max_execution_time'));

		// If the destination directory doesn't exist 
		// we need to create it
		if (!file_exists(dirname($file)))
		{
			if ($this->app->filesystem->folder->create(dirname($file)) == false)
			{
				return false;
			}
		}

		// try putting content into the file
		$file 	= $this->app->filesystem->path->clean($file);
		$ret 	= is_int(file_put_contents($file, $buffer)) ? true : false;

		return $ret;
	}

	/**
	 * Wrapper for the standard file_exists function
	 *
	 * @param   string  $file  File path
	 * @return  boolean  True if path is a file
	 * @since   1.0.0
	 */
	public function exists($file)
	{
		return is_file($this->app->filesystem->path->clean($file));
	}
	
	/**
	 * Wrapper for the standard filemtime function
	 *
	 * @param   string  	$file  File path
	 * @return  mixed	  False if not found time if found
	 * @since   1.0.0
	 */
	public function filetime($file)
	{ 
		if ( $this->exists($file) )
		{
			return filemtime($this->app->filesystem->path->clean($file));
		}
		
		return false;
	}

	/**
	 * Returns the name, without any path.
	 *
	 * @param   string  $file  File path
	 * @return  string  filename
	 * @since   1.0.0
	 */
	public function getName($file)
	{
		if ( $this->exists($file) ) {
			return basename($file);
		}
		
		return false;
	}
}
