<?php
/**
 * @package     Bookzo.Platform
 * @subpackage  Form
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Form Field class for the Joomla Platform.
 *
 * Provides a pop up date picker linked to a button.
 * Optionally may be filtered to use user's or server's time zone.
 *
 * @package     Bookzo.Platform
 * @subpackage  Form
 * @since       11.1
 */
class BFormFieldCalendar extends BFormField
{
	/**
	 * The form field type.
	 *
	 * @var    string
	 * @since  11.1
	 */
	protected $type = 'Calendar';

	/**
	 * The allowable maxlength of calendar field.
	 *
	 * @var    integer
	 * @since  3.2
	 */
	protected $maxlength;

	/**
	 * The format of date and time.
	 *
	 * @var    integer
	 * @since  3.2
	 */
	protected $format;
	
	/**
	 * The format to show in popup.
	 *
	 * @var    integer
	 * @since  3.2
	 */
	protected $calendarformat;
	
	/**
	 * The max date for the calendar.
	 *
	 * @var    integer
	 * @since  3.2
	 */
	protected $maxdate;
	
	/**
	 * The mindate for the calendar.
	 *
	 * @var    integer
	 * @since  3.2
	 */
	protected $mindate;

	/**
	 * The filter.
	 *
	 * @var    integer
	 * @since  3.2
	 */
	protected $filter;

	/**
	 * Method to get certain otherwise inaccessible properties from the form field object.
	 *
	 * @param   string  $name  The property name for which to the the value.
	 *
	 * @return  mixed  The property value or null.
	 *
	 * @since   3.2
	 */
	public function __get($name)
	{
		switch ($name)
		{
			case 'maxlength':
			case 'format':
			case 'calendarformat':
			case 'mindate':
			case 'maxdate':
			case 'filter':
				return $this->$name;
		}

		return parent::__get($name);
	}

	/**
	 * Method to set certain otherwise inaccessible properties of the form field object.
	 *
	 * @param   string  $name   The property name for which to the the value.
	 * @param   mixed   $value  The value of the property.
	 *
	 * @return  void
	 *
	 * @since   3.2
	 */
	public function __set($name, $value)
	{
		switch ($name)
		{
			case 'maxlength':
				$value = (int) $value;

			case 'format':
			case 'calendarformat':
			case 'mindate':
			case 'maxdate':
			case 'filter':
				$this->$name = (string) $value;
				break;

			default:
				parent::__set($name, $value);
		}
	}

	/**
	 * Method to attach a JForm object to the field.
	 *
	 * @param   SimpleXMLElement  $element  The SimpleXMLElement object representing the <field /> tag for the form field object.
	 * @param   mixed             $value    The form field value to validate.
	 * @param   string            $group    The field name group control value. This acts as as an array container for the field.
	 *                                      For example if the field has name="foo" and the group value is set to "bar" then the
	 *                                      full field name would end up being "bar[foo]".
	 *
	 * @return  boolean  True on success.
	 *
	 * @see     BFormField::setup()
	 * @since   3.2
	 */
	public function setup(SimpleXMLElement $element, $value, $group = null)
	{
		$return = parent::setup($element, $value, $group);

		if ($return)
		{
			$this->maxlength 		= (int) $this->element['maxlength'] ? (int) $this->element['maxlength'] : 45;
			$this->format    		= (string) $this->element['format'] ? (string) $this->element['format'] : 'Y-m-d';
			$this->calendarformat   = (string) $this->element['calendarformat'] ? (string) $this->element['calendarformat'] : 'YYYY-MM-DD';
			$this->filter    		= (string) $this->element['filter'] ? (string) $this->element['filter'] : 'USER_UTC';
			$this->maxdate    		= (string) $this->element['maxdate'] ? (string) $this->element['maxdate'] : '';
			$this->mindate    		= (string) $this->element['mindate'] ? (string) $this->element['mindate'] : '';
		}

		return $return;
	}

	/**
	 * Method to get the field input markup.
	 *
	 * @return  string  The field input markup.
	 *
	 * @since   11.1
	 */
	protected function getInput()
	{
		// Translate placeholder text
		$hint 		= $this->translateHint ? BText::_($this->hint) : $this->hint;
		
		// Build settings
		$settings 	= array();
		empty($this->mindate)      		? null : $settings['mindate'] 		= $this->mindate;
		empty($this->maxdate)      		? null : $settings['maxdate'] 		= $this->maxdate;
		empty($this->format)      		? null : $settings['format'] 		= $this->format;
		empty($this->calendarformat) 	? null : $settings['calendarformat']= $this->calendarformat;

		// Build the attributes array.
		$attributes = array();
		empty($this->size)      ? null : $attributes['size'] 		 = $this->size;
		empty($this->maxlength) ? null : $attributes['maxlength'] 	 = $this->maxlength;
		empty($this->class)     ? null : $attributes['class'] 		 = $this->class;
		!$this->readonly        ? null : $attributes['readonly'] 	 = true;
		!$this->disabled        ? null : $attributes['disabled']	 = false;
		empty($this->onchange)  ? null : $attributes['onchange'] 	 = $this->onchange;
		empty($hint)            ? null : $attributes['placeholder']  = $hint;
		$this->autocomplete     ? null : $attributes['autocomplete'] = 'off';
		!$this->autofocus       ? null : $attributes['autofocus'] 	 = '';

		if ($this->required)
		{
			$attributes['required'] 		= '';
		}
		
		// Handle the special case for "now".
		if (strtoupper($this->value) == 'NOW')
		{
			$this->value = Bookzo::getInstance()->date->create()->format( BText::_($this->format) );
		}
		
		if ((int) $this->value)
		{
			// Get a date object based on the correct timezone.
			$date 		 = Bookzo::getInstance()->date->create($this->value, 'UTC');
			$this->value = $date->format(BText::_($this->format), true, false);
		}
		
		return BHtml::_('calendar', $this->value, $this->name, $this->id, $settings, $attributes);
	}
}
