<?php
/**
 * @package     Bookzo.Platform
 * @subpackage  Form
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Form Field class for the Joomla Platform.
 * Supports a one line text field.
 *
 * @package     Bookzo.Platform
 * @subpackage  Form
 * @link        http://www.w3.org/TR/html-markup/input.text.html#input.text
 * @since       11.1
 */
class BFormFieldText extends BFormField
{
	/**
	 * The form field type.
	 *
	 * @var    string
	 *
	 * @since  11.1
	 */
	protected $type = 'Text';

	/**
	 * The allowable maxlength of the field.
	 *
	 * @var    integer
	 * @since  3.2
	 */
	protected $maxLength;

	/**
	 * Method to get certain otherwise inaccessible properties from the form field object.
	 *
	 * @param   string  $name  The property name for which to the the value.
	 *
	 * @return  mixed  The property value or null.
	 *
	 * @since   3.2
	 */
	public function __get($name)
	{
		switch ($name)
		{
			case 'maxLength':
				return $this->$name;
		}

		return parent::__get($name);
	}

	/**
	 * Method to set certain otherwise inaccessible properties of the form field object.
	 *
	 * @param   string  $name   The property name for which to the the value.
	 * @param   mixed   $value  The value of the property.
	 *
	 * @return  void
	 *
	 * @since   3.2
	 */
	public function __set($name, $value)
	{
		switch ($name)
		{
			case 'maxLength':
				$this->maxLength = (int) $value;
				break;

			default:
				parent::__set($name, $value);
		}
	}

	/**
	 * Method to attach a JForm object to the field.
	 *
	 * @param   SimpleXMLElement  $element  The SimpleXMLElement object representing the <field /> tag for the form field object.
	 * @param   mixed             $value    The form field value to validate.
	 * @param   string            $group    The field name group control value. This acts as as an array container for the field.
	 *                                      For example if the field has name="foo" and the group value is set to "bar" then the
	 *                                      full field name would end up being "bar[foo]".
	 *
	 * @return  boolean  True on success.
	 *
	 * @see     BFormField::setup()
	 * @since   3.2
	 */
	public function setup(SimpleXMLElement $element, $value, $group = null)
	{
		$result = parent::setup($element, $value, $group);

		if ($result == true)
		{
			$this->maxLength = (int) $this->element['maxlength'];
		}

		return $result;
	}

	/**
	 * Method to get the field input markup.
	 *
	 * @return  string  The field input markup.
	 *
	 * @since   11.1
	 */
	protected function getInput()
	{		
		// Translate placeholder text
		$hint = $this->translateHint ? BText::_($this->hint) : $this->hint;

		// Initialize some field attributes.
		$datastring = $this->getAttribute('datastring');
		$size       = !empty($this->size) 		? ' size="' . $this->size . '"' 			: '';
		$maxLength  = !empty($this->maxLength) 	? ' maxlength="' . $this->maxLength . '"' 	: '';
		$class      = !empty($this->class) 		? ' class="' . $this->class . '"' 			: '';
		$pattern    = !empty($this->pattern) 	? ' pattern="' . $this->pattern . '"' 		: '';
		$required   = $this->required 			? ' required aria-required="true"' 			: '';
		$hint       = $hint 					? ' placeholder="' . $hint . '"' 			: '';		
		$readonly	= $this->readonly 			? ' readonly' : '';
		$disabled   = $this->disabled 			? ' disabled' : '';
		$dataattr 	= !empty($datastring)  		? ' ' . $datastring .' ' : '';

		// Initialize JavaScript field attributes.
		$onchange 	= !empty($this->onchange) ? ' onchange="' . $this->onchange . '"' : '';

		// render the field
		$html	  = '<input type="text" name="' . $this->name . '" id="' . $this->id . '" value="'
			. htmlspecialchars($this->value, ENT_COMPAT, 'UTF-8') . '"' . $class . $size . $disabled . $readonly 
			. $hint . $onchange . $maxLength . $required . $pattern . $dataattr . ' />';

		return $html;
	}
}
