<?php
/**
 * @package     Joomla.Libraries
 * @subpackage  HTML
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Utility class for all HTML drawing classes
 *
 * @since  1.5
 */
abstract class BHtml
{
	/**
	 * An array to hold method references
	 *
	 * @var    array
	 * @since  1.6
	 */
	protected static $registry = array();

	/**
	 * Method to extract a key
	 *
	 * @param   string  $key  The name of helper method to load, (prefix).(class).function
	 *                        prefix and class are optional and can be used to load custom html helpers.
	 *
	 * @return  array  Contains lowercase key, prefix, file, function.
	 *
	 * @since   1.6
	 */
	protected static function extract($key)
	{
		$key 	= preg_replace('#[^A-Z0-9_\.]#i', '', $key);

		// Check to see whether we need to load a helper file
		$parts 	= explode('.', $key);
		$prefix = (count($parts) == 3 ? array_shift($parts) : 'BHtml');
		$file 	= (count($parts) == 2 ? array_shift($parts) : '');
		$func 	= array_shift($parts);

		return array(strtolower($prefix . '.' . $file . '.' . $func), $prefix, $file, $func);
	}
	
	/**
	 * Class loader method
	 *
	 * Additional arguments may be supplied and are passed to the sub-class.
	 * Additional include paths are also able to be specified for third-party use
	 *
	 * @param   string  $key  The name of helper method to load, (prefix).(class).function
	 *                        prefix and class are optional and can be used to load custom
	 *                        html helpers.
	 *
	 * @return  mixed  JHtml::call($function, $args) or False on error
	 *
	 * @since   1.5
	 * @throws  InvalidArgumentException
	 */
	public static function _($key)
	{
		list($key, $prefix, $file, $func) = static::extract($key);

		if (array_key_exists($key, static::$registry))
		{
			$function = static::$registry[$key];
			$args 	  = func_get_args();

			// Remove function name from arguments
			array_shift($args);

			return static::call($function, $args);
		}

		$class = $prefix . ucfirst($file);
		
		if (!class_exists($class))
		{
			throw new BRuntimeException(sprintf('%s %s not found.', $prefix, $file), 500);
		}

		$toCall = array($class, $func);

		if (is_callable($toCall))
		{
			static::register($key, $toCall);
			$args = func_get_args();

			// Remove function name from arguments
			array_shift($args);

			return static::call($toCall, $args);
		}
		else
		{
			throw new BRuntimeException(sprintf('%s::%s not found.', $class, $func), 500);
		}
	}

	/**
	 * Registers a function to be called with a specific key
	 *
	 * @param   string  $key       The name of the key
	 * @param   string  $function  Function or method
	 *
	 * @return  boolean  True if the function is callable
	 *
	 * @since   1.6
	 */
	public static function register($key, $function)
	{
		list($key) = static::extract($key);

		if (is_callable($function))
		{
			static::$registry[$key] = $function;

			return true;
		}

		return false;
	}

	/**
	 * Function caller method
	 *
	 * @param   callable  $function  Function or method to call
	 * @param   array     $args      Arguments to be passed to function
	 *
	 * @return  mixed   Function result or false on error.
	 *
	 * @see     http://php.net/manual/en/function.call-user-func-array.php
	 * @since   1.6
	 * @throws  InvalidArgumentException
	 */
	protected static function call($function, $args)
	{
		if (!is_callable($function))
		{
			throw new InvalidArgumentException('Function not supported', 500);
		}

		// PHP 5.3 workaround
		$temp = array();

		foreach ($args as &$arg)
		{
			$temp[] = &$arg;
		}

		return call_user_func_array($function, $temp);
	}
	
	/**
	 * Displays a calendar control field
	 *
	 * @param   string  $value    The date value
	 * @param   string  $name     The name of the text field
	 * @param   string  $id       The id of the text field
	 * @param   string  $format   The date format
	 * @param   mixed   $attribs  Additional HTML attributes
	 *
	 * @return  string  HTML markup for a calendar field
	 *
	 * @since   1.5
	 */
	public static function calendar($value, $name, $id, $settings = null, $attribs = null)
	{
		static $done;

		if ($done === null)
		{
			$done = array();
		}
		
		$bookzo	  = Bookzo::getInstance();
		$settings = $bookzo->registry->create($settings);

		// turn attribs into string
		if (is_array($attribs))
		{
			$attribs['class'] = isset($attribs['class']) ? $attribs['class'] : 'input-medium';
			$attribs['class'] = trim($attribs['class'] . ' hasTooltip');

			$attribs = $bookzo->array->toString($attribs);
		}

		## NOTE - Tooltips should be handled
		//static::_('bootstrap.tooltip');

		// Format value when not nulldate ('0000-00-00 00:00:00'), otherwise blank it as it would result in 1970-01-01.
		if ($value && $value != '0000-00-00 00:00:00')
		{
			$tz = date_default_timezone_get();
			$inputvalue = $bookzo->date->create($value, 'UTC')->format(BText::_($settings->get('format', 'Y-m-d')));
			date_default_timezone_set($tz);
		}
		else
		{
			$inputvalue = '';
		}

		// Load the calendar behavior
		static::_('behavior.calendar');
		
		$months = array(
					BText::_('JANUARY'), BText::_('FEBRUARY'), BText::_('MARCH'), BText::_('APRIL'), BText::_('MAY'), BText::_('JUNE'), 
					BText::_('JULY'), BText::_('AUGUST'), BText::_('SEPTEMBER'), BText::_('OCTOBER'), BText::_('NOVEMBER'), BText::_('DECEMBER')
					);
		$weeks 	= array(
						BText::_('SUN'), BText::_('MON'), BText::_('TUE'), BText::_('WED'), BText::_('THU'), BText::_('FRI'), BText::_('SAT')
					);
	

		/*
		return '<div' . $div_class . '>'
				. '<input type="text" title="' . ($inputvalue ? static::_('date', $value, null, null) : '')
				. '" name="' . $name . '" id="' . $id . '" value="' . htmlspecialchars($inputvalue, ENT_COMPAT, 'UTF-8') . '" ' . $attribs . ' />'
				. '<button type="button" class="btn" id="' . $id . '_img"' . $btn_style . '><span class="icon-calendar"></span></button>'
			. '</div>';
			
		*/
		
		## NOTE - work on this one
		$calsettings	= array();
		$calsettings[]  = "i18n: { months : ['".implode("','", $months)."'], weekdays: ['".implode("','", $weeks)."']}";
		
		if ( $calendarformat = $settings->get('calendarformat', 'YYYY-MM-DD') ) {
			$calsettings[]  = "format:'".BText::_( $calendarformat )."'";
		}
		
		if ( $mindate = $settings->get('mindate') ) {
			$calsettings[]  = "minDate:'". $bookzo->date->create($mindate)->format($settings->get('format', 'Y-m-d'))."'";
		}
		
		if ( $maxdate = $settings->get('maxdate') ) {
			$calsettings[]  = "maxDate:'". $bookzo->date->create($maxdate)->format($settings->get('format', 'Y-m-d'))."'";
		}		
		
		return '<input type="text" name="' . $name . '" id="' . $id . '" value="' . htmlspecialchars($inputvalue, ENT_COMPAT, 'UTF-8') . '" ' . $attribs
				 . ' data-bz-datepicker="{'.implode(',', $calsettings).'} ">';
	}
	
/**
	 * Converts a double colon seperated string or 2 separate strings to a string ready for bootstrap tooltips
	 *
	 * @param   string  $title      The title of the tooltip (or combined '::' separated string).
	 * @param   string  $content    The content to tooltip.
	 * @param   int     $translate  If true will pass texts through BText.
	 * @param   int     $escape     If true will pass texts through htmlspecialchars.
	 *
	 * @return  string  The tooltip string
	 *
	 * @since   3.1.2
	 */
	public static function tooltipText($title = '', $content = '', $translate = 1, $escape = 1)
	{
		// Initialise return value.
		$result = '';

		// Don't process empty strings
		if ($content != '' || $title != '')
		{
			// Split title into title and content if the title contains '::' (old Mootools format).
			if ($content == '' && !(strpos($title, '::') === false))
			{
				list($title, $content) = explode('::', $title, 2);
			}

			// Pass texts through BText if required.
			if ($translate)
			{
				$title = BText::_($title);
				$content = BText::_($content);
			}

			// Use only the content if no title is given.
			if ($title == '')
			{
				$result = $content;
			}
			// Use only the title, if title and text are the same.
			elseif ($title == $content)
			{
				$result = '<strong>' . $title . '</strong>';
			}
			// Use a formatted string combining the title and content.
			elseif ($content != '')
			{
				$result = '<strong>' . $title . '</strong><br />' . $content;
			}
			else
			{
				$result = $title;
			}

			// Escape everything, if required.
			if ($escape)
			{
				$result = htmlspecialchars($result);
			}
		}

		return $result;
	}
}
