<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/
defined('_BOOKZO') or die;

if ( !defined('_BQQ_') ) {
	define('_BQQ_', '"');
}

/**
 * COPY OF JLANGUAGE. I LIKE THE WAY JOOMLA HANLES LANGUAGES
 * 
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BLanguage {

	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	public $app;
	
	/**
	 * Array of JLanguage objects
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected static $languages = array();

	/**
	 * The default language, used when a language file in the requested language does not exist.
	 *
	 * @var    string
	 * @since  11.1
	 */
	protected $default = 'nl';

	/**
	 * An array of orphaned text.
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected $orphans = array();

	/**
	 * Array holding the language metadata.
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected $metadata = null;

	/**
	 * Array holding the language locale or boolean null if none.
	 *
	 * @var    array|boolean
	 * @since  11.1
	 */
	protected $locale = null;

	/**
	 * The language to load.
	 *
	 * @var    string
	 * @since  11.1
	 */
	protected $lang = null;
	
	/**
	 * The bookzo language id
	 *
	 * @var    string
	 * @since  11.1
	 */
	protected $langid = null;

	/**
	 * A nested array of language files that have been loaded
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected $paths = array();

	/**
	 * List of language files that are in error state
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected $errorfiles = array();

	/**
	 * Translations
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected $strings = array();

	/**
	 * An array of used text, used during debugging.
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected $used = array();

	/**
	 * Counter for number of loads.
	 *
	 * @var    integer
	 * @since  11.1
	 */
	protected $counter = 0;

	/**
	 * An array used to store overrides.
	 *
	 * @var    array
	 * @since  11.1
	 */
	protected $override = array();
	
	/**
	 * Name of the transliterator function for this language.
	 *
	 * @var    string
	 * @since  11.1
	 */
	protected $transliterator = null;

	/**
	 * Name of the pluralSuffixesCallback function for this language.
	 *
	 * @var    callable
	 * @since  11.1
	 */
	protected $pluralSuffixesCallback = null;

	/**
	 * Constructor activating the default information of the language.
	 *
	 * @param   string   $lang   The language
	 *
	 * @since   11.1
	 */
	public function __construct($app, $lang = null, $langid = null)
	{
		$this->app		= $app;
		$this->strings 	= array();

		if ($lang == null)
		{
			$lang = $this->default;
		}

		$this->setLanguage($lang, $langid);

		// load override file if present
		if (($filename = $this->app->path->path('languages:'.$lang.'/'.$lang. '.override.ini')) && ($contents = $this->parse($filename)))
		{

			if (is_array($contents))
			{
				// Sort the underlying heap by key values to optimize merging
				ksort($contents, SORT_STRING);
				$this->override = $contents;
			}

			unset($contents);
		}
		
		// get the localize file and map functions to the tranlator
		$class = 'BLanguage_'. str_replace('-', '_', $lang . '_Localise');
		
		if ( $file = $this->app->path->path('languages:'.$lang.'/'.$lang.'.localise.php') ) 
		{
			if ( class_exists($class) ) 
			{
				if (method_exists($class, 'transliterate'))
				{
					$this->transliterator = array($class, 'transliterate');
				}
	
				if (method_exists($class, 'getPluralSuffixes'))
				{
					$this->pluralSuffixesCallback = array($class, 'getPluralSuffixes');
				}
			}
		}
	
		$this->load();
	}

	/**
	 * Returns a language object.
	 *
	 * @param   string   $lang   The language to use.
	 *
	 * @return  JLanguage  The Language object.
	 *
	 * @since   11.1
	 */
	public static function getInstance($app, $lang, $langid)
	{
		if (!isset(self::$languages[$lang]))
		{
			self::$languages[$lang] = new BLanguage($app, $lang, $langid);
		}

		return self::$languages[$lang];
	}

	/**
	 * Translate function, mimics the php gettext (alias _) function.
	 *
	 * The function checks if $jsSafe is true, then if $interpretBackslashes is true.
	 *
	 * @param   string   $string                The string to translate
	 * @param   boolean  $jsSafe                Make the result javascript safe
	 * @param   boolean  $interpretBackSlashes  Interpret \t and \n
	 *
	 * @return  string  The translation of the string
	 *
	 * @since   11.1
	 */
	public function _($string, $jsSafe = false, $interpretBackSlashes = true)
	{
		// Detect empty string
		if ($string == '')
		{
			return '';
		}

		$key = strtoupper($string);

		if (isset($this->strings[$key]))
		{
			$string = $this->strings[$key];
		}


		if ($jsSafe)
		{
			// Javascript filter
			$string = addslashes($string);
		}
		elseif ($interpretBackSlashes)
		{
			// Interpret \n and \t characters
			$string = str_replace(array('\\\\', '\t', '\n'), array("\\", "\t", "\n"), $string);
		}

		return $string;
	}

	/**
	 * Transliterate function
	 *
	 * This method processes a string and replaces all accented UTF-8 characters by unaccented
	 * ASCII-7 "equivalents".
	 *
	 * @param   string  $string  The string to transliterate.
	 *
	 * @return  string  The transliteration of the string.
	 *
	 * @since   11.1
	 */
	public function transliterate($string)
	{
		if ($this->transliterator !== null)
		{
			return call_user_func($this->transliterator, $string);
		}
		
		$string = BLanguageTransliterate::utf8_latin_to_ascii($string);
		## NOTE - we should see what to do with utf8 lanuages and jstring
		//$string = JString::strtolower($string);
		$string = strtolower($string);

		return $string;
	}
	
	/**
	 * Returns an array of suffixes for plural rules.
	 *
	 * @param   integer  $count  The count number the rule is for.
	 *
	 * @return  array    The array of suffixes.
	 *
	 * @since   11.1
	 */
	public function getPluralSuffixes($count)
	{
		if ($this->pluralSuffixesCallback !== null)
		{
			return call_user_func($this->pluralSuffixesCallback, $count);
		}
		else
		{
			return array((string) $count);
		}
	}

	/**
	 * Checks if a language exists.
	 *
	 * This is a simple, quick check for the directory that should contain language files for the given user.
	 *
	 * @param   string  $lang      Language to check.
	 * @param   string  $basePath  Optional path to check.
	 *
	 * @return  boolean  True if the language exists.
	 *
	 * @since   11.1
	 */
	public function exists($lang)
	{
		static $paths = array();

		// Return false if no language was specified
		if (!$lang)
		{
			return false;
		}

		// get filename
		$filename   = $lang.'/'.$lang.'.ini';
		
		// Return previous check results if it exists
		if (isset($paths[$filename]))
		{
			return $paths[$filename];
		}
		
		// find the file to see if it exists
		$paths[$filename] = (bool) $this->app->path->path('languages:'.$filename);

		return $paths[$filename];
	}

	/**
	 * Loads a single language file and appends the results to the existing strings
	 *
	 * @param   string   $basePath   The basepath to use.
	 * @param   string   $extension  The extension for which a language file should be loaded.
	 * @param   string   $lang       The language to load, default null for the current language.
	 * @param   boolean  $reload     Flag that will force a language to be reloaded if set to true.
	 * @param   boolean  $default    Flag that force the default language to be loaded if the current does not exist.
	 *
	 * @return  boolean  True if the file has successfully loaded.
	 *
	 * @since   11.1
	 */
	public function load($extension = 'root', $lang = null, $reload = false, $default = true, $adddefault = null)
	{
		// determine lang
		if (!$lang)
		{
			$lang = $this->lang;
		}
		
		// get the filename 
		$suffix   	= $extension == 'root' ? '' : '.'.$extension;
		$filename	= $lang.'/'.$lang.$suffix.'.ini';

		if (isset($this->paths[$filename]) && !$reload)
		{
			// This file has already been tested for loading.
			$result = $this->paths[$filename];
		}
		else
		{
			// Load the language file
			$result = $this->loadLanguage($this->app->path->path('languages:'.$filename));

			// Check whether there was a problem with loading the file
			if ($result === false && $default)
			{
				// No strings, so either file doesn't exist or the file is invalid
				$oldFilename = $filename;

				// Check the standard file name
				$filename	= $this->default.'/'.$this->default.$suffix.'.ini';

				// If the one we tried is different than the new name, try again
				if ($oldFilename != $filename)
				{
					$result = $this->loadLanguage($this->app->path->path('languages:'.$filename), false);
				}
			}
			
			$this->paths[$filename] = $result;
		}
		
		return $result;
	}

	/**
	 * Loads a language file.
	 *
	 * This method will not note the successful loading of a file - use load() instead.
	 *
	 * @param   string  $filename   The name of the file.
	 *
	 * @return  boolean  True if new strings have been added to the language
	 *
	 * @see     JLanguage::load()
	 * @since   11.1
	 */
	protected function loadLanguage($filename)
	{
		$this->counter++;

		$result = false;
		$strings = false;

		if ( is_file($filename) ) 
		{
			if (file_exists($filename))
			{
				$strings = $this->parse($filename);
			}
			
			
	
			if ($strings)
			{
				if (is_array($strings))
				{
					// Sort the underlying heap by key values to optimize merging
					ksort($strings, SORT_STRING);
					$this->strings = array_merge($this->strings, $strings);
				}
	
				if (is_array($strings) && count($strings))
				{
					// Do not bother with ksort here.  Since the originals were sorted, PHP will already have chosen the best heap.
					$this->strings = array_merge($this->strings, $this->override);
					$result = true;
				}
			}
		}

		return $result;
	}
	
	public function addOverrides($contents)
	{
		if (is_array($contents))
		{
			$this->override = array_merge($this->override, $contents);
			
			// Sort the underlying heap by key values to optimize merging
			ksort($this->override, SORT_STRING);
		}
	}

	/**
	 * Parses a language file.
	 *
	 * @param   string  $filename  The name of the file.
	 *
	 * @return  array  The array of parsed strings.
	 *
	 * @since   11.1
	 */
	protected function parse($filename)
	{
		$contents 	= file_get_contents($filename);
		$contents 	= str_replace('_BQQ_', '"\""', $contents);
		$strings 	= @parse_ini_string($contents);

		if (!is_array($strings))
		{
			$strings = array();
		}

		return $strings;
	}

	/**
	 * Get a metadata language property.
	 *
	 * @param   string  $property  The name of the property.
	 * @param   mixed   $default   The default value.
	 *
	 * @return  mixed  The value of the property.
	 *
	 * @since   11.1
	 */
	public function get($property, $default = null)
	{
		if (isset($this->metadata[$property]))
		{
			return $this->metadata[$property];
		}

		return $default;
	}

	/**
	 * Getter for Name.
	 *
	 * @return  string  Official name element of the language.
	 *
	 * @since   11.1
	 */
	public function getName()
	{
		return $this->metadata['name'];
	}
	
	/**
	 * Getter for ID.
	 *
	 * @return  string  Official name element of the language.
	 *
	 * @since   11.1
	 */
	public function getId()
	{
		return (int) $this->langid;
	}
	
	/**
	 * Getter for CODE.
	 *
	 * @return  string  Official name element of the language.
	 *
	 * @since   11.1
	 */
	public function getCode()
	{
		return (string) $this->lang;
	}

	/**
	 * Get a list of language files that have been loaded.
	 *
	 * @param   string  $extension  An optional extension name.
	 *
	 * @return  array
	 *
	 * @since   11.1
	 */
	public function getPaths()
	{
		return $this->paths;
	}

	/**
	 * Get a list of language files that are in error state.
	 *
	 * @return  array
	 *
	 * @since   11.1
	 */
	public function getErrorFiles()
	{
		return $this->errorfiles;
	}

	/**
	 * Getter for the language tag (as defined in RFC 3066)
	 *
	 * @return  string  The language tag.
	 *
	 * @since   11.1
	 */
	public function getTag()
	{
		return $this->metadata['tag'];
	}

	/**
	 * Get the RTL property.
	 *
	 * @return  boolean  True is it an RTL language.
	 *
	 * @since   11.1
	 */
	public function isRTL()
	{
		return (bool) $this->metadata['rtl'];
	}

	/**
	 * Get the default language code.
	 *
	 * @return  string  Language code.
	 *
	 * @since   11.1
	 */
	public function getDefault()
	{
		return $this->default;
	}

	/**
	 * Set the default language code.
	 *
	 * @param   string  $lang  The language code.
	 *
	 * @return  string  Previous value.
	 *
	 * @since   11.1
	 */
	public function setDefault($lang)
	{
		$previous = $this->default;
		$this->default = $lang;

		return $previous;
	}

	/**
	 * Get the list of orphaned strings if being tracked.
	 *
	 * @return  array  Orphaned text.
	 *
	 * @since   11.1
	 */
	public function getOrphans()
	{
		return $this->orphans;
	}

	/**
	 * Get the list of used strings.
	 *
	 * Used strings are those strings requested and found either as a string or a constant.
	 *
	 * @return  array  Used strings.
	 *
	 * @since   11.1
	 */
	public function getUsed()
	{
		return $this->used;
	}

	/**
	 * Determines is a key exists.
	 *
	 * @param   string  $string  The key to check.
	 *
	 * @return  boolean  True, if the key exists.
	 *
	 * @since   11.1
	 */
	public function hasKey($string)
	{
		$key = strtoupper($string);

		return isset($this->strings[$key]);
	}

	/**
	 * Returns a associative array holding the metadata.
	 *
	 * @param   string  $lang  The name of the language.
	 *
	 * @return  mixed  If $lang exists return key/value pair with the language metadata, otherwise return NULL.
	 *
	 * @since   11.1
	 */
	public function getMetadata($lang)
	{
		$result = null;

		if ($file = $this->app->path->path('languages:'.$lang.'/'.$lang.'.xml'))
		{
			$result = $this->parseXMLLanguageFile($file);
		}

		if (empty($result))
		{
			return null;
		}

		return $result;
	}

	/**
	 * Returns a list of known languages for an area
	 *
	 * @param   string  $basePath  The basepath to use
	 *
	 * @return  array  key/value pair with the language file and real name.
	 *
	 * @since   11.1
	 */
	public function getKnownLanguages()
	{
		$knownLanguages = $this->parseLanguageFiles();

		return $knownLanguages;
	}

	/**
	 * Set the language attributes to the given language.
	 *
	 * Once called, the language still needs to be loaded using JLanguage::load().
	 *
	 * @param   string  $lang  Language code.
	 *
	 * @return  string  Previous value.
	 *
	 * @since   11.1
	 */
	public function setLanguage($lang, $langid = 1)
	{
		$previous 		= $this->lang;
		$this->lang 	= $lang;
		$this->langid 	= $langid;
		$this->metadata = $this->getMetadata($this->lang);

		return $previous;
	}

	/**
	 * Get the language locale based on current language.
	 *
	 * @return  array  The locale according to the language.
	 *
	 * @since   11.1
	 */
	public function getLocale()
	{
		if (!isset($this->locale))
		{
			$locale = str_replace(' ', '', isset($this->metadata['locale']) ? $this->metadata['locale'] : '');

			if ($locale)
			{
				$this->locale = explode(',', $locale);
			}
			else
			{
				$this->locale = false;
			}
		}

		return $this->locale;
	}

	/**
	 * Get the first day of the week for this language.
	 *
	 * @return  integer  The first day of the week according to the language
	 *
	 * @since   11.1
	 */
	public function getFirstDay()
	{
		return (int) (isset($this->metadata['firstDay']) ? $this->metadata['firstDay'] : 0);
	}

	/**
	 * Get the weekends days for this language.
	 *
	 * @return  string  The weekend days of the week separated by a comma according to the language
	 *
	 * @since   3.2
	 */
	public function getWeekEnd()
	{
		return (isset($this->metadata['weekEnd']) && $this->metadata['weekEnd']) ? $this->metadata['weekEnd'] : '0,6';
	}

	/**
	 * Searches for language directories within a certain base dir.
	 *
	 * @param   string  $dir  directory of files.
	 *
	 * @return  array  Array holding the found languages as filename => real name pairs.
	 *
	 * @since   11.1
	 */
	public static function parseLanguageFiles()
	{
		$languages 	= array();
		$files		= $this->app->path->files('languages:', true, "/^([-_A-Za-z]*)\.xml$/");

		foreach ($files as $file)
		{
			$langs    = array();
			$fileName = basename($file);

			try
			{
				$metadata = $this->parseXMLLanguageFile($file);

				if ($metadata)
				{
					$lang = str_replace('.xml', '', $fileName);
					$langs[$lang] = $metadata;
				}

				$languages = array_merge($languages, $langs);
			}
			catch (RuntimeException $e)
			{
			}
		}

		return $languages;
	}

	/**
	 * Parse XML file for language information.
	 *
	 * @param   string  $path  Path to the XML files.
	 *
	 * @return  array  Array holding the found metadata as a key => value pair.
	 *
	 * @since   11.1
	 * @throws  RuntimeException
	 */
	public function parseXMLLanguageFile($path)
	{
		if (!is_readable($path))
		{
			throw new BRuntimeException('File not found or not readable');
		}

		// Try to load the file
		$xml = simplexml_load_file($path);

		if (!$xml)
		{
			return null;
		}

		// Check that it's a metadata file
		if ((string) $xml->getName() != 'metafile')
		{
			return null;
		}

		$metadata = array();

		foreach ($xml->metadata->children() as $child)
		{
			$metadata[$child->getName()] = (string) $child;
		}

		return $metadata;
	}

}