<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Base View class
 *
 * @package Framework.Classes
 */
class BModel extends BObject {
	
	/**
	 * Instance container.
	 *
	 * @var    JControllerLegacy
	 * @since  12.2
	 */
	protected static $instances;
	
	/**
	 * Indicates if the internal state has been set
	 *
	 * @var    boolean
	 * @since  12.2
	 */
	protected $state_set = null;
	
	/**
	 * The model (base) name
	 *
	 * @var    string
	 * @since  12.2
	 */
	protected $name;
	
	/**
	 * A state object
	 *
	 * @var    string
	 * @since  12.2
	 */
	protected $state;
	
	
	
	/**
	 * Constructor
	 *
	 * @param   array  $config  An array of configuration options (name, state, dbo, table_path, ignore_request).
	 *
	 * @since   12.2
	 * @throws  Exception
	 */
	public function __construct($config = array())
	{
		parent::__construct();
		
		// Set the model name
		if (empty($this->name))
		{
			$this->name = $this->getName();
		}

		// Set the model state
		$this->state 		= new BObject();
		$this->state_set 	= true;
		$this->populateState();
	}
	
	/**
	 * Returns a Model object, always creating it
	 *
	 * @param   string  $type    The model type to instantiate
	 * @param   string  $prefix  Prefix for the model class name. Optional.
	 * @param   array   $config  Configuration array for model. Optional.
	 *
	 * @return  mixed   A model object or false on failure
	 *
	 * @since   12.2
	 */
	public static function getInstance($type, $config = array())
	{
		// make sure instances is array 
		if (!isset(self::$instances))
		{
			self::$instances = array();
		}
		
		// find correct model if not yet found
		$app		= Bookzo::getInstance();
		$type 		= strtolower(preg_replace('/[^A-Z0-9_\.-]/i', '', $type));
		$class 		= 'BzModel'. ucfirst($type);

		// find correct model if not yet found
		if (!isset(self::$instances[$class]))
		{ 
			// try finding correct model
			## NOTE - Do this in 1 time
			if ( $path = $app->path->path('models:'.$type.'.php') ) {
				$app->loader->register($class, 'models:'.$type.'.php');
				self::$instances[$class] = new $class($config);
			}
			else {
				self::$instances[$class] = new BModelDefault($config);
			}
		}

		// return the instance
		return self::$instances[$class];
	}


	/**
	 * Method to get the model name
	 *
	 * The model name. By default parsed using the classname or it can be set
	 * by passing a $config['name'] in the class constructor
	 *
	 * @return  string  The name of the model
	 *
	 * @since   12.2
	 * @throws  Exception
	 */
	public function getName()
	{
		if (empty($this->name))
		{
			$r = null;

			if (!preg_match('/Model(.*)/i', get_class($this), $r))
			{
				throw new BModelException('JLIB_APPLICATION_ERROR_MODEL_GET_NAME', 500);
			}

			$this->name = strtolower($r[1]);
		}

		return $this->name;
	}
	
	/**
	 * Method to auto-populate the model state.
	 *
	 * This method should only be called once per instantiation and is designed
	 * to be called on the first call to the getState() method unless the model
	 * configuration flag to ignore the request is set.
	 *
	 * @return  void
	 *
	 * @note    Calling getState in this method will result in recursion.
	 * @since   12.2
	 */
	protected function populateState()
	{
	}
	
	/**
	 * Method to set model state variables
	 *
	 * @param   string  $property  The name of the property.
	 * @param   mixed   $value     The value of the property to set or null.
	 *
	 * @return  mixed  The previous value of the property or null if not set.
	 *
	 * @since   12.2
	 */
	public function setState($property, $value = null)
	{
		return $this->state->set($property, $value);
	}
	
	/**
	 * Method to get model state variables
	 *
	 * @param   string  $property  Optional parameter name
	 * @param   mixed   $default   Optional default value
	 *
	 * @return  object  The property where specified, the state object where omitted
	 *
	 * @since   12.2
	 */
	public function getState($property = null, $default = null)
	{
		if (!$this->state_set)
		{
			// Protected method to auto-populate the model state.
			$this->populateState();

			// Set the model state set flag to true.
			$this->state_set = true;
		}

		return $property === null ? $this->state : $this->state->get($property, $default);
	}

}

/**
 * Dedicated Exception for the AppView class
 *
 * @see AppView
 */
class BModelException extends BzException {}