<?php
/**
 * @package     Joomla.Libraries
 * @subpackage  Pagination
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Pagination Class. Provides a common interface for content pagination for the Joomla! CMS.
 *
 * @package     Joomla.Libraries
 * @subpackage  Pagination
 * @since       1.5
 */
class BPagination
{
	/**
	 * @var    integer  The record number to start displaying from.
	 * @since  1.5
	 */
	public $app = null;
	
	/**
	 * @var    integer  The record number to start displaying from.
	 * @since  1.5
	 */
	public $limitstart = null;

	/**
	 * @var    integer  Number of rows to display per page.
	 * @since  1.5
	 */
	public $limit = null;

	/**
	 * @var    integer  Total number of rows.
	 * @since  1.5
	 */
	public $total = null;

	/**
	 * @var    integer  Prefix used for request variables.
	 * @since  1.6
	 */
	public $prefix = null;

	/**
	 * @var    integer  Value pagination object begins at
	 * @since  3.0
	 */
	public $pagesStart;

	/**
	 * @var    integer  Value pagination object ends at
	 * @since  3.0
	 */
	public $pagesStop;

	/**
	 * @var    integer  Current page
	 * @since  3.0
	 */
	public $pagesCurrent;

	/**
	 * @var    integer  Total number of pages
	 * @since  3.0
	 */
	public $pagesTotal;
	
	/**
	 * @var    integer  Total number of pages
	 * @since  3.0
	 */
	public $currentPage;

	/**
	 * @var    boolean  View all flag
	 * @since  3.0
	 */
	protected $viewall = false;

	/**
	 * Additional URL parameters to be added to the pagination URLs generated by the class.  These
	 * may be useful for filters and extra values when dealing with lists and GET requests.
	 *
	 * @var    array
	 * @since  3.0
	 */
	protected $additionalUrlParams = array();

	/**
	 * Constructor.
	 *
	 * @param   integer  $total       The total number of items.
	 * @param   integer  $limitstart  The offset of the item to start at.
	 * @param   integer  $limit       The number of items to display per page.
	 * @param   string   $prefix      The prefix used for request variables.
	 *
	 * @since   1.5
	 */
	public function __construct($total, $limitstart, $limit, $prefix = '')
	{
		// Value/type checking.
		$this->app 			= Bookzo::getInstance();
		$this->total 		= (int) $total;
		$this->limitstart 	= (int) max($limitstart, 0);
		$this->limit 		= (int) max($limit, 0);
		$this->prefix 		= $prefix;
		$this->renderer 	= $this->app->renderer->partial('pagination');
		$this->currentPage  = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : '';
		
		$paths = $this->renderer->getPaths();
		
		if ($this->limit > $this->total)
		{
			$this->limitstart = 0;
		}

		if (!$this->limit)
		{
			$this->limit = $total;
			$this->limitstart = 0;
		}

		/*
		 * If limitstart is greater than total (i.e. we are asked to display records that don't exist)
		 * then set limitstart to display the last natural page of results
		 */
		if ($this->limitstart > $this->total - $this->limit)
		{
			$this->limitstart = max(0, (int) (ceil($this->total / $this->limit) - 1) * $this->limit);
		}

		// Set the total pages and current page values.
		if ($this->limit > 0)
		{
			$this->pagesTotal = ceil($this->total / $this->limit);
			$this->pagesCurrent = ceil(($this->limitstart + 1) / $this->limit);
		}

		// Set the pagination iteration loop values.
		$displayedPages = 10;
		$this->pagesStart = $this->pagesCurrent - ($displayedPages / 2);

		if ($this->pagesStart < 1)
		{
			$this->pagesStart = 1;
		}

		if ($this->pagesStart + $displayedPages > $this->pagesTotal)
		{
			$this->pagesStop = $this->pagesTotal;

			if ($this->pagesTotal < $displayedPages)
			{
				$this->pagesStart = 1;
			}
			else
			{
				$this->pagesStart = $this->pagesTotal - $displayedPages + 1;
			}
		}
		else
		{
			$this->pagesStop = $this->pagesStart + $displayedPages - 1;
		}

		// If we are viewing all records set the view all flag to true.
		if ($limit == 0)
		{
			$this->viewall = true;
		}
	}

	/**
	 * Method to set an additional URL parameter to be added to all pagination class generated
	 * links.
	 *
	 * @param   string  $key    The name of the URL parameter for which to set a value.
	 * @param   mixed   $value  The value to set for the URL parameter.
	 *
	 * @return  mixed  The old value for the parameter.
	 *
	 * @since   1.6
	 */
	public function setAdditionalUrlParam($key, $value)
	{
		// Get the old value to return and set the new one for the URL parameter.
		$result = isset($this->additionalUrlParams[$key]) ? $this->additionalUrlParams[$key] : null;

		// If the passed parameter value is null unset the parameter, otherwise set it to the given value.
		if ($value === null)
		{
			unset($this->additionalUrlParams[$key]);
		}
		else
		{
			$this->additionalUrlParams[$key] = $value;
		}

		return $result;
	}

	/**
	 * Method to get an additional URL parameter (if it exists) to be added to
	 * all pagination class generated links.
	 *
	 * @param   string  $key  The name of the URL parameter for which to get the value.
	 *
	 * @return  mixed  The value if it exists or null if it does not.
	 *
	 * @since   1.6
	 */
	public function getAdditionalUrlParam($key)
	{
		$result = isset($this->additionalUrlParams[$key]) ? $this->additionalUrlParams[$key] : null;

		return $result;
	}

	/**
	 * Return the rationalised offset for a row with a given index.
	 *
	 * @param   integer  $index  The row index
	 *
	 * @return  integer  Rationalised offset for a row with a given index.
	 *
	 * @since   1.5
	 */
	public function getRowOffset($index)
	{
		return $index + 1 + $this->limitstart;
	}

	/**
	 * Return the pagination data object, only creating it if it doesn't already exist.
	 *
	 * @return  object   Pagination data object.
	 *
	 * @since   1.5
	 */
	public function getData()
	{
		static $data;

		if (!is_object($data))
		{
			$data = $this->_buildDataObject();
		}

		return $data;
	}

	/**
	 * Create and return the pagination pages counter string, ie. Page 2 of 4.
	 *
	 * @return  string   Pagination pages counter string.
	 *
	 * @since   1.5
	 */
	public function getPagesCounter()
	{
		$html = null;

		if ($this->pagesTotal > 1)
		{
			$html .= BText::sprintf('BPAGINATION_PAGE_CURRENT_OF_TOTAL', $this->pagesCurrent, $this->pagesTotal);
		}

		return $html;
	}

	/**
	 * Create and return the pagination result set counter string, e.g. Results 1-10 of 42
	 *
	 * @return  string   Pagination result set counter string.
	 *
	 * @since   1.5
	 */
	public function getResultsCounter()
	{
		$html = null;
		$fromResult = $this->limitstart + 1;

		// If the limit is reached before the end of the list.
		if ($this->limitstart + $this->limit < $this->total)
		{
			$toResult = $this->limitstart + $this->limit;
		}
		else
		{
			$toResult = $this->total;
		}

		// If there are results found.
		if ($this->total > 0)
		{
			$msg = BText::sprintf('BPAGINATION_RESULTS_OF', $fromResult, $toResult, $this->total);
			$html .= "\n" . $msg;
		}
		else
		{
			$html .= "\n" . BText::_('BPAGINATION_NO_RECORDS_FOUND');
		}

		return $html;
	}

	/**
	 * Create and return the pagination page list string, ie. Previous, Next, 1 2 3 ... x.
	 *
	 * @return  string  Pagination page list string.
	 *
	 * @since   1.5
	 */
	public function getPagesLinks( $type = 'default' )
	{
		// Build the page navigation list.
		$data = $this->_buildDataObject();
		$type = $this->app->config->get('template.pagination_type', $type);

		$list = array();
		$list['prefix'] = $this->prefix;

		$itemOverride = false;
		$listOverride = false;

		// Build the select list
		if ( $type != 'short' ) 
		{
			$list['start'] = $this->item($data->start, $data->start->base !== null, 'start' );
		}
		
		// previous
		if ( $type == 'long' || ( $type == 'short' && $data->previous->base !== null ) ) {
			if ( $type == 'short' ) {
				$data->previous->text = BText::sprintf('BPREV_NR', $this->limit);
			}
			$list['previous'] = $this->item($data->previous, $data->previous->base !== null, 'prev' );
		}

		if ( $type != 'short' ) 
		{
			// Make sure it exists
			$list['pages'] = array();
			
			// for default pagination we want only the pages surrounding current
			$min 		   = $this->pagesCurrent - 1;
			$max 		   = $this->pagesCurrent + 1;
			$showfirstdots = true;
			$showlastdots  = true;
			
			if ( $type == 'default' ) {
				if ( $this->pagesCurrent == 1 ) {
					$min = 1;
					$max = 3;
				}
				elseif ( $this->pagesCurrent >= $this->pagesStop ) {
					$min = $this->pagesCurrent - 2;
					$max = $this->pagesCurrent;
					$min = $min < 1 ? 1 : $min;
				}
			}
	
			// pages
			foreach ($data->pages as $i => $page)
			{
				if ( $type == 'default' ) 
				{
					if ( $i == $min - 1 || $i == $max + 1) {
						$page->text = '...';
						$list['pages'][$i] = $this->item($page, true, 'page' );
					}
				}
				
				if ( $type != 'default' || ( $i >= $min && $i <= $max)  ) {
					$list['pages'][$i] = $this->item($page, true, 'page' . ($i == $this->pagesCurrent ? ' current' : '') );
				}
			}
		}

		// next button
		if ( $type == 'long' || ( $type == 'short' && $data->next->base !== null ) ) {
			if ( $type == 'short' ) {
				$remaining = $this->total - ($this->limit * $this->pagesCurrent);
				$nr = $remaining > $this->limit ? $this->limit : $remaining;
				$data->next->text = BText::sprintf('BNEXT_NR', $nr);
			}
			$list['next'] = $this->item($data->next, $data->next->base !== null, 'next' );
		}

		// end button
		if ( $type != 'short' ) 
		{
			$list['end'] = $this->item($data->end, $data->end->base !== null, 'next' );
		}

		if ($this->total > $this->limit)
		{
			return $this->renderList($list, $type);
		}
		else
		{
			return '';
		}
	}

	## NOTE - We should have a function for a total pagination with counter etc
	/**
	 * Get the pagination links
	 *
	 * @param   string  $layoutId  Layout to render the links
	 * @param   array   $options   Optional array with settings for the layout
	 *
	 * @return  string  Pagination links.
	 *
	 * @since   3.3
	 */
	 /*
	public function getPaginationLinks($layoutId = 'joomla.pagination.links', $options = array())
	{
		// Allow to receive a null layout
		$layoutId = (null === $layoutId) ? 'joomla.pagination.links' : $layoutId;

		$app = JFactory::getApplication();

		$list = array(
			'prefix'       => $this->prefix,
			'limit'        => $this->limit,
			'limitstart'   => $this->limitstart,
			'total'        => $this->total,
			'limitfield'   => $this->getLimitBox(),
			'pagescounter' => $this->getPagesCounter(),
			'pages'        => $this->getPaginationPages()
		);

		return JLayoutHelper::render($layoutId, array('list' => $list, 'options' => $options));
	}
	*/

	/**
	 * Create and return the pagination page list string, ie. Previous, Next, 1 2 3 ... x.
	 *
	 * @return  string  Pagination page list string.
	 *
	 * @since   3.3
	 */
	public function getPaginationPages()
	{
		$list = array();

		if ($this->total > $this->limit)
		{
			// Build the page navigation list.
			$data = $this->_buildDataObject();

			// All
			$list['all']['active'] = (null !== $data->all->base);
			$list['all']['data']   = $data->all;

			// Start
			$list['start']['active'] = (null !== $data->start->base);
			$list['start']['data']   = $data->start;

			// Previous link
			$list['previous']['active'] = (null !== $data->previous->base);
			$list['previous']['data']   = $data->previous;

			// Make sure it exists
			$list['pages'] = array();

			foreach ($data->pages as $i => $page)
			{
				$list['pages'][$i]['active'] = (null !== $page->base);
				$list['pages'][$i]['data']   = $page;
			}

			$list['next']['active'] = (null !== $data->next->base);
			$list['next']['data']   = $data->next;

			$list['end']['active'] = (null !== $data->end->base);
			$list['end']['data']   = $data->end;
		}

		return $list;
	}

	/**
	 * Creates a dropdown box for selecting how many records to show per page.
	 *
	 * @return  string  The HTML for the limit # input box.
	 *
	 * @since   1.5
	 */
	public function getLimitBox()
	{
		$limits = array();

		// Make the option list.
		for ($i = 5; $i <= 30; $i += 5)
		{
			$limits[] = BHtml::_('select.option', "$i");
		}

		$limits[] = BHtml::_('select.option', '50', 	BText::_('J50'));
		$limits[] = BHtml::_('select.option', '100', 	BText::_('J100'));
		$limits[] = BHtml::_('select.option', '0', 		BText::_('JALL'));

		$selected = $this->viewall ? 0 : $this->limit;

		// Build the select list.
		$html = BHtml::_(
			'select.genericlist',
			$limits,
			$this->prefix . 'limit',
			'class="inputbox input-mini" size="1" onchange="this.form.submit()"',
			'value',
			'text',
			$selected
		);

		return $html;
	}

	/**
	 * Create the html for a list footer
	 *
	 * @param   array  $list  Pagination list data structure.
	 *
	 * @return  string  HTML for a list start, previous, next,end
	 *
	 * @since   1.5
	 */
	protected function renderList($list, $type = 'default')
	{
		// try override first
		if ( $this->renderer->exists('list') ) {
			return $this->renderer->render('list', compact('list', 'type'));
		}
		
		// Reverse output rendering for right-to-left display.
		$html = '<ul class="bz-pagination bz-pagination-type-'.$type.'>';
		if ( isset($list['start']) ) {
			$html .= '<li class="pagination-start">' . $list['start'] . '</li>';
		}
		if ( isset($list['previous']) ) {
			$html .= '<li class="pagination-prev">' . $list['previous'] . '</li>';
		}

		foreach ($list['pages'] as $page)
		{
			$html .= '<li class="pagination-page">' . $page . '</li>';
		}

		if ( isset($list['next']) ) {
			$html .= '<li class="pagination-next">' . $list['next'] . '</li>';
		}
		if ( isset($list['end']) ) {
			$html .= '<li class="pagination-end">' . $list['end'] . '</li>';
		}
		$html .= '</ul>';

		return $html;
	}

	/**
	 * Method to create an active pagination link to the item
	 *
	 * @param   JPaginationObject  $item  The object with which to make an active link.
	 *
	 * @return  string  HTML link
	 *
	 * @since   1.5
	 */
	protected function item(BPaginationObject $item, $active = false, $class = '')
	{
		if ( $this->renderer->exists('item') ) {
			return $this->renderer->render('item', compact('item', 'active', 'class'));
		}
		
		if ( $active ) {
			return '<a href="' . $item->link . '" class="pagenav '.$class.'">' . $item->text . '</a>';
		}
		else {
			return '<span class="pagenav '.$class.'">' . $item->text . '</a>';
		}
	}

	/**
	 * Create and return the pagination data object.
	 *
	 * @return  object  Pagination data object.
	 *
	 * @since   1.5
	 */
	protected function _buildDataObject()
	{
		$data = new stdClass;

		// Build the additional URL parameters string.
		$params = '';

		if (!empty($this->additionalUrlParams))
		{
			foreach ($this->additionalUrlParams as $key => $value)
			{
				$params .= '&' . $key . '=' . $value;
			}
		}

		$data->all = new BPaginationObject(BText::_('BPAGINATION_VIEW_ALL'), $this->prefix);

		## NOTE - !!! important !!!
		## Look into the routing here


		if (!$this->viewall)
		{
			$data->all->base = '0';
			$data->all->link = $this->buildLink($params . '&' . $this->prefix . 'limitstart=');
			
		}

		// Set the start and previous data objects.
		$data->start 	= new BPaginationObject(BText::_('BPAGINATION_START'), $this->prefix);
		$data->previous = new BPaginationObject(BText::_('BPREV'), $this->prefix);

		if ($this->pagesCurrent > 1)
		{
			$page = $this->pagesCurrent > 1 ? ($this->pagesCurrent - 2) * $this->limit : 0;

			// Set the empty for removal from route
			// @todo remove code: $page = $page == 0 ? '' : $page;
			$data->start->base = '0';
			$data->start->link = $this->buildLink($params . '&' . $this->prefix . 'limitstart=0');
			$data->previous->base = $page;
			$data->previous->link = $this->buildLink($params . '&' . $this->prefix . 'limitstart=' . $page);
		}

		// Set the next and end data objects.
		$data->next = new BPaginationObject(BText::_('BNEXT'), $this->prefix);
		$data->end = new BPaginationObject(BText::_('BPAGINATION_END'), $this->prefix);

		if ($this->pagesCurrent < $this->pagesTotal)
		{
			$next = $this->pagesCurrent * $this->limit;
			$end = ($this->pagesTotal - 1) * $this->limit;

			$data->next->base = $next;
			$data->next->link = $this->buildLink($params . '&' . $this->prefix . 'limitstart=' . $next);
			$data->end->base = $end;
			$data->end->link = $this->buildLink($params . '&' . $this->prefix . 'limitstart=' . $end);
		}

		$data->pages = array();
		$stop = $this->pagesStop;

		for ($i = $this->pagesStart; $i <= $stop; $i++)
		{
			$offset = ($i - 1) * $this->limit;

			$data->pages[$i] = new BPaginationObject($i, $this->prefix);

			if (!$this->viewall)
			{
				$data->pages[$i]->base = $offset;
				$data->pages[$i]->link = $this->buildLink($params . '&' . $this->prefix . 'limitstart=' . $offset);
				$data->pages[$i]->active = $i == $this->pagesCurrent;
			}
		}

		return $data;
	}
	
	protected function buildLink($link)
	{
		$link   = substr($link, 0, 1) == '&' ? substr($link, 1) : $link;
		$return = strpos($this->currentPage, '?') !== false ? $this->currentPage . '&' . $link : $this->currentPage . '?' . $link;
		
		
		// fix double parts 
		$parts	= explode('?', $return);
		
		if ( count($parts) == 2 ) 
		{
			$tmp  = array();
			$vars = explode('&', $parts[1]);
			
			foreach ( $vars as $var ) 
			{
				$varparts = explode('=', $var);
				
				if ( count($varparts) == 2 ) 
				{
					$tmp[$varparts[0]] = $varparts[0] .'='. $varparts[1];
				}
			}
			
			return $parts[0] . '?' .  implode('&', $tmp);
		}
		else {
			return $return;
		}
	}
}
