<?php
/**
* This is the dynamic loader for the library. It checks whether you have
* the mbstring extension available and includes relevant files
* on that basis, falling back to the native (as in written in PHP) version
* if mbstring is unavailabe.
*
* It's probably easiest to use this, if you don't want to understand
* the dependencies involved, in conjunction with PHP versions etc. At
* the same time, you might get better performance by managing loading
* yourself. The smartest way to do this, bearing in mind performance,
* is probably to "load on demand" - i.e. just before you use these
* functions in your code, load the version you need.
*
* It makes sure the the following functions are available;
* utf8_strlen, utf8_strpos, utf8_strrpos, utf8_substr,
* utf8_strtolower, utf8_strtoupper
* Other functions in the ./native directory depend on these
* six functions being available
* @package utf8
*/

defined('_BOOKZO') or die;

/**
* If string overloading is active, it will break many of the
* native implementations. mbstring.func_overload must be set
* to 0, 1 or 4 in php.ini (string overloading disabled).
* Also need to check we have the correct internal mbstring
* encoding
*/
if ( extension_loaded('mbstring')) {
    if ( ini_get('mbstring.func_overload') & MB_OVERLOAD_STRING ) {
        trigger_error('String functions are overloaded by mbstring',E_USER_ERROR);
    }
    mb_internal_encoding('UTF-8');
}

/**
* Check whether PCRE has been compiled with UTF-8 support
*/
$UTF8_ar = array();
if ( preg_match('/^.{1}$/u',"ñ",$UTF8_ar) != 1 ) {
    trigger_error('PCRE is not compiled with UTF-8 support',E_USER_ERROR);
}
unset($UTF8_ar);

abstract class BStringUtf8 implements iBStringUtf8
{
	protected static $class;
	
	/**
	 * Do we use native of mbstring class
	 *
	 * @var    array
	 * @since  1.0
	 */
	protected static function getClass()
	{
		if ( is_null(static::$class) ) 
		{
			if ( function_exists('mb_substr') ) {
				static::$class = 'BStringUtf8Mbstring';
			}
			else {
				static::$class = 'BStringUtf8Native';
			}
		}
		
		return static::$class;
	}
	
	/**
	* UTF-8 aware substr_replace.
	* Note: requires utf8_substr to be loaded
	* @see http://www.php.net/substr_replace
	* @see utf8_strlen
	* @see utf8_substr
	*/
	public static function utf8_substr_replace($str, $repl, $start , $length = NULL ) {
		
		preg_match_all('/./us', $str, $ar);
		preg_match_all('/./us', $repl, $rar);
		if( $length === NULL ) {
			$class 	= static::getClass();
			$length = $class::utf8_strlen($str);
		}
		array_splice( $ar[0], $start, $length, $rar[0] );
		return join('',$ar[0]);
	}
	
	/**
	* UTF-8 aware alternative to ucwords
	* Uppercase the first character of each word in a string
	* Note: requires utf8_substr_replace and utf8_strtoupper
	* @param string
	* @return string with first char of each word uppercase
	* @see http://www.php.net/ucwords
	* @package utf8
	*/
	public static function utf8_ucwords($str) {
		// Note: [\x0c\x09\x0b\x0a\x0d\x20] matches;
		// form feeds, horizontal tabs, vertical tabs, linefeeds and carriage returns
		// This corresponds to the definition of a "word" defined at http://www.php.net/ucwords
		$pattern = '/(^|([\x0c\x09\x0b\x0a\x0d\x20]+))([^\x0c\x09\x0b\x0a\x0d\x20]{1})[^\x0c\x09\x0b\x0a\x0d\x20]*/u';
		return preg_replace_callback($pattern, array('BStringUtf8', 'utf8_ucwords_callback'),$str);
	}
	
	/**
	* Callback function for preg_replace_callback call in utf8_ucwords
	* You don't need to call this yourself
	* @param array of matches corresponding to a single word
	* @return string with first char of the word in uppercase
	* @see utf8_ucwords
	* @see utf8_strtoupper
	* @package utf8
	*/
	public static function utf8_ucwords_callback($matches) {
		$leadingws 	= $matches[2];
		$class		= static::getClass();
		$ucfirst 	= $class::utf8_strtoupper($matches[3]);
		$ucword 	= static::utf8_substr_replace(ltrim($matches[0]),$ucfirst,0,1);
		return $leadingws . $ucword;
	}
	
	/**
	* UTF-8 aware alternative to ucfirst
	* Make a string's first character uppercase
	* Note: requires utf8_strtoupper
	* @param string
	* @return string with first character as upper case (if applicable)
	* @see http://www.php.net/ucfirst
	* @see utf8_strtoupper
	* @package utf8
	*/
	public static function utf8_ucfirst($str){
		$class = static::getClass();
		switch ( $class::utf8_strlen($str) ) {
			case 0:
				return '';
			break;
			case 1:
				return $class::utf8_strtoupper($str);
			break;
			default:
				preg_match('/^(.{1})(.*)$/us', $str, $matches);
				return $class::utf8_strtoupper($matches[1]).$matches[2];
			break;
		}
	}
	
	/**
	* UTF-8 aware replacement for ltrim()
	* Note: you only need to use this if you are supplying the charlist
	* optional arg and it contains UTF-8 characters. Otherwise ltrim will
	* work normally on a UTF-8 string
	* @author Andreas Gohr <andi@splitbrain.org>
	* @see http://www.php.net/ltrim
	* @see http://dev.splitbrain.org/view/darcs/dokuwiki/inc/utf8.php
	* @return string
	* @package utf8
	*/
	public static function utf8_ltrim( $str, $charlist = FALSE ) {
		if($charlist === FALSE) return ltrim($str);
	
		//quote charlist for use in a characterclass
		$charlist = preg_replace('!([\\\\\\-\\]\\[/^])!','\\\${1}',$charlist);
	
		return preg_replace('/^['.$charlist.']+/u','',$str);
	}
	
	//---------------------------------------------------------------
	/**
	* UTF-8 aware replacement for rtrim()
	* Note: you only need to use this if you are supplying the charlist
	* optional arg and it contains UTF-8 characters. Otherwise rtrim will
	* work normally on a UTF-8 string
	* @author Andreas Gohr <andi@splitbrain.org>
	* @see http://www.php.net/rtrim
	* @see http://dev.splitbrain.org/view/darcs/dokuwiki/inc/utf8.php
	* @return string
	* @package utf8
	*/
	public static function utf8_rtrim( $str, $charlist = FALSE ) {
		if($charlist === FALSE) return rtrim($str);
	
		//quote charlist for use in a characterclass
		$charlist = preg_replace('!([\\\\\\-\\]\\[/^])!','\\\${1}',$charlist);
	
		return preg_replace('/['.$charlist.']+$/u','',$str);
	}
	
	//---------------------------------------------------------------
	/**
	* UTF-8 aware replacement for trim()
	* Note: you only need to use this if you are supplying the charlist
	* optional arg and it contains UTF-8 characters. Otherwise trim will
	* work normally on a UTF-8 string
	* @author Andreas Gohr <andi@splitbrain.org>
	* @see http://www.php.net/trim
	* @see http://dev.splitbrain.org/view/darcs/dokuwiki/inc/utf8.php
	* @return string
	* @package utf8
	*/
	public static function utf8_trim( $str, $charlist = FALSE ) {
		if($charlist === FALSE) return trim($str);
		return static::utf8_ltrim(static::utf8_rtrim($str, $charlist), $charlist);
	}
	

	/**
	* UTF-8 aware alternative to strspn
	* Find length of initial segment matching mask
	* Note: requires utf8_strlen and utf8_substr (if start, length are used)
	* @param string
	* @return int
	* @see http://www.php.net/strspn
	* @package utf8
	*/
	public static function utf8_strspn($str, $mask, $start = NULL, $length = NULL) {
	
		$mask = preg_replace('!([\\\\\\-\\]\\[/^])!','\\\${1}',$mask);
		$class = static::getClass();
	
		// Fix for $start but no $length argument.
		if ($start !== null && $length === null) {
			$length = $class::utf8_strlen($str);
		}
	
		if ( $start !== NULL || $length !== NULL ) {
			$str = $class::utf8_substr($str, $start, $length);
		}
	
		preg_match('/^['.$mask.']+/u',$str, $matches);
	
		if ( isset($matches[0]) ) {
			return $class::utf8_strlen($matches[0]);
		}
	
		return 0;
	}
	
	/**
	* UTF-8 aware alternative to strrev
	* Reverse a string
	* @param string UTF-8 encoded
	* @return string characters in string reverses
	* @see http://www.php.net/strrev
	* @package utf8
	*/
	public static function utf8_strrev($str){
		preg_match_all('/./us', $str, $ar);
		return join('',array_reverse($ar[0]));
	}
	
	/**
	* UTF-8 aware alternative to stristr
	* Find first occurrence of a string using case insensitive comparison
	* Note: requires utf8_strtolower
	* @param string
	* @param string
	* @return int
	* @see http://www.php.net/strcasecmp
	* @see utf8_strtolower
	* @package utf8
	*/
	public static function utf8_stristr($str, $search) {
	
		if ( strlen($search) == 0 ) {
			return $str;
		}
	
		$class = static::getClass();
		$lstr = $class::utf8_strtolower($str);
		$lsearch = $class::utf8_strtolower($search);
		//JOOMLA SPECIFIC FIX - BEGIN
		preg_match('/^(.*)'.preg_quote($lsearch, '/').'/Us',$lstr, $matches);
		//JOOMLA SPECIFIC FIX - END
	
		if ( count($matches) == 2 ) {
			return substr($str, strlen($matches[1]));
		}
	
		return FALSE;
	}
	
	/**
	* UTF-8 aware alternative to strcspn
	* Find length of initial segment not matching mask
	* Note: requires utf8_strlen and utf8_substr (if start, length are used)
	* @param string
	* @return int
	* @see http://www.php.net/strcspn
	* @see utf8_strlen
	* @package utf8
	*/
	public static function utf8_strcspn($str, $mask, $start = NULL, $length = NULL) {
	
		if ( empty($mask) || strlen($mask) == 0 ) {
			return NULL;
		}
	
		$class = static::getClass();
		$mask = preg_replace('!([\\\\\\-\\]\\[/^])!','\\\${1}',$mask);
	
		if ( $start !== NULL || $length !== NULL ) {
			$str = $class::utf8_substr($str, $start, $length);
		}
	
		preg_match('/^[^'.$mask.']+/u',$str, $matches);
	
		if ( isset($matches[0]) ) {
			return $class::utf8_strlen($matches[0]);
		}
	
		return 0;
	
	}
	
	/**
	* UTF-8 aware alternative to strcasecmp
	* A case insensivite string comparison
	* Note: requires utf8_strtolower
	* @param string
	* @param string
	* @return int
	* @see http://www.php.net/strcasecmp
	* @see utf8_strtolower
	* @package utf8
	*/
	public static function utf8_strcasecmp($strX, $strY) {
		$class 	= static::getClass();
		$strX 	= $class::utf8_strtolower($strX);
		$strY 	= $class::utf8_strtolower($strY);
		return strcmp($strX, $strY);
	}
	
	/**
	* UTF-8 aware alternative to str_split
	* Convert a string to an array
	* Note: requires utf8_strlen to be loaded
	* @param string UTF-8 encoded
	* @param int number to characters to split string by
	* @return string characters in string reverses
	* @see http://www.php.net/str_split
	* @see utf8_strlen
	* @package utf8
	*/
	public static function utf8_str_split($str, $split_len = 1) {
	
		if ( !preg_match('/^[0-9]+$/',$split_len) || $split_len < 1 ) {
			return FALSE;
		}
	
		$class 	= static::getClass();
		$len 	= $class::utf8_strlen($str);
		if ( $len <= $split_len ) {
			return array($str);
		}
	
		preg_match_all('/.{'.$split_len.'}|[^\x00]{1,'.$split_len.'}$/us', $str, $ar);
		return $ar[0];
	
	}
	
	/**
	* Replacement for str_pad. $padStr may contain multi-byte characters.
	*
	* @author Oliver Saunders <oliver (a) osinternetservices.com>
	* @param string $input
	* @param int $length
	* @param string $padStr
	* @param int $type ( same constants as str_pad )
	* @return string
	* @see http://www.php.net/str_pad
	* @see utf8_substr
	* @package utf8
	*/
	public static function utf8_str_pad($input, $length, $padStr = ' ', $type = STR_PAD_RIGHT) {
		$class = static::getClass();
		$inputLen = $class::utf8_strlen($input);
		if ($length <= $inputLen) {
			return $input;
		}
	
		$padStrLen = $class::utf8_strlen($padStr);
		$padLen = $length - $inputLen;
	
		if ($type == STR_PAD_RIGHT) {
			$repeatTimes = ceil($padLen / $padStrLen);
			return $class::utf8_substr($input . str_repeat($padStr, $repeatTimes), 0, $length);
		}
	
		if ($type == STR_PAD_LEFT) {
			$repeatTimes = ceil($padLen / $padStrLen);
			return $class::utf8_substr(str_repeat($padStr, $repeatTimes), 0, floor($padLen)) . $input;
		}
	
		if ($type == STR_PAD_BOTH) {
	
			$padLen/= 2;
			$padAmountLeft = floor($padLen);
			$padAmountRight = ceil($padLen);
			$repeatTimesLeft = ceil($padAmountLeft / $padStrLen);
			$repeatTimesRight = ceil($padAmountRight / $padStrLen);
	
			$paddingLeft = $class::utf8_substr(str_repeat($padStr, $repeatTimesLeft), 0, $padAmountLeft);
			$paddingRight = $class::utf8_substr(str_repeat($padStr, $repeatTimesRight), 0, $padAmountLeft);
			return $paddingLeft . $input . $paddingRight;
		}
	
		trigger_error('utf8_str_pad: Unknown padding type (' . $type . ')',E_USER_ERROR);
	}
	
	/**
	* UTF-8 aware alternative to str_ireplace
	* Case-insensitive version of str_replace
	* Note: requires utf8_strtolower
	* Note: it's not fast and gets slower if $search / $replace is array
	* Notes: it's based on the assumption that the lower and uppercase
	* versions of a UTF-8 character will have the same length in bytes
	* which is currently true given the hash table to strtolower
	* @param string
	* @return string
	* @see http://www.php.net/str_ireplace
	* @see utf8_strtolower
	* @package utf8
	*/
	public static function utf8_ireplace($search, $replace, $str, $count = NULL){
	
		if ( !is_array($search) ) {
	
			$class = static::getClass();
			$slen = strlen($search);
			if ( $slen == 0 ) {
				return $str;
			}
	
			$lendif = strlen($replace) - strlen($search);
			$search = $class::utf8_strtolower($search);
	
			$search = preg_quote($search, '/');
			$lstr = $class::utf8_strtolower($str);
			$i = 0;
			$matched = 0;
			while ( preg_match('/(.*)'.$search.'/Us',$lstr, $matches) ) {
				if ( $i === $count ) {
					break;
				}
				$mlen = strlen($matches[0]);
				$lstr = substr($lstr, $mlen);
				$str = substr_replace($str, $replace, $matched+strlen($matches[1]), $slen);
				$matched += $mlen + $lendif;
				$i++;
			}
			return $str;
	
		} else {
	
			foreach ( array_keys($search) as $k ) {
	
				if ( is_array($replace) ) {
	
					if ( array_key_exists($k,$replace) ) {
	
						$str = static::utf8_ireplace($search[$k], $replace[$k], $str, $count);
	
					} else {
	
						$str = static::utf8_ireplace($search[$k], '', $str, $count);
	
					}
	
				} else {
	
					$str = static::utf8_ireplace($search[$k], $replace, $str, $count);
	
				}
			}
			return $str;
	
		}
	
	}
	
	/**
	* UTF-8 aware alternative to ord
	* Returns the unicode ordinal for a character
	* @param string UTF-8 encoded character
	* @return int unicode ordinal for the character
	* @see http://www.php.net/ord
	* @see http://www.php.net/manual/en/function.ord.php#46267
	*/
	public static function utf8_ord($chr) {
	
		$ord0 = ord($chr);
	
		if ( $ord0 >= 0 && $ord0 <= 127 ) {
			return $ord0;
		}
	
		if ( !isset($chr{1}) ) {
			trigger_error('Short sequence - at least 2 bytes expected, only 1 seen');
			return FALSE;
		}
	
		$ord1 = ord($chr{1});
		if ( $ord0 >= 192 && $ord0 <= 223 ) {
			return ( $ord0 - 192 ) * 64
				+ ( $ord1 - 128 );
		}
	
		if ( !isset($chr{2}) ) {
			trigger_error('Short sequence - at least 3 bytes expected, only 2 seen');
			return FALSE;
		}
		$ord2 = ord($chr{2});
		if ( $ord0 >= 224 && $ord0 <= 239 ) {
			return ($ord0-224)*4096
				+ ($ord1-128)*64
					+ ($ord2-128);
		}
	
		if ( !isset($chr{3}) ) {
			trigger_error('Short sequence - at least 4 bytes expected, only 3 seen');
			return FALSE;
		}
		$ord3 = ord($chr{3});
		if ($ord0>=240 && $ord0<=247) {
			return ($ord0-240)*262144
				+ ($ord1-128)*4096
					+ ($ord2-128)*64
						+ ($ord3-128);
	
		}
	
		if ( !isset($chr{4}) ) {
			trigger_error('Short sequence - at least 5 bytes expected, only 4 seen');
			return FALSE;
		}
		$ord4 = ord($chr{4});
		if ($ord0>=248 && $ord0<=251) {
			return ($ord0-248)*16777216
				+ ($ord1-128)*262144
					+ ($ord2-128)*4096
						+ ($ord3-128)*64
							+ ($ord4-128);
		}
	
		if ( !isset($chr{5}) ) {
			trigger_error('Short sequence - at least 6 bytes expected, only 5 seen');
			return FALSE;
		}
		if ($ord0>=252 && $ord0<=253) {
			return ($ord0-252) * 1073741824
				+ ($ord1-128)*16777216
					+ ($ord2-128)*262144
						+ ($ord3-128)*4096
							+ ($ord4-128)*64
								+ (ord($chr{5})-128);
		}
	
		if ( $ord0 >= 254 && $ord0 <= 255 ) {
			trigger_error('Invalid UTF-8 with surrogate ordinal '.$ord0);
			return FALSE;
		}
	
	}
	
	/**
	* Tests whether a string contains only 7bit ASCII bytes.
	* You might use this to conditionally check whether a string
	* needs handling as UTF-8 or not, potentially offering performance
	* benefits by using the native PHP equivalent if it's just ASCII e.g.;
	*
	* <code>
	* if ( utf8_is_ascii($someString) ) {
	*     // It's just ASCII - use the native PHP version
	*     $someString = strtolower($someString);
	* } else {
	*     $someString = utf8_strtolower($someString);
	* }
	* </code>
	*
	* @param string
	* @return boolean TRUE if it's all ASCII
	* @package utf8
	* @see utf8_is_ascii_ctrl
	*/
	public static function utf8_is_ascii($str) {
		// Search for any bytes which are outside the ASCII range...
		return (preg_match('/(?:[^\x00-\x7F])/',$str) !== 1);
	}
	
	/**
	* Tests whether a string contains only 7bit ASCII bytes with device
	* control codes omitted. The device control codes can be found on the
	* second table here: http://www.w3schools.com/tags/ref_ascii.asp
	*
	* @param string
	* @return boolean TRUE if it's all ASCII without device control codes
	* @package utf8
	* @see utf8_is_ascii
	*/
	public static function utf8_is_ascii_ctrl($str) {
		if ( strlen($str) > 0 ) {
			// Search for any bytes which are outside the ASCII range,
			// or are device control codes
			return (preg_match('/[^\x09\x0A\x0D\x20-\x7E]/',$str) !== 1);
		}
		return FALSE;
	}
	
	/**
	* Strip out all non-7bit ASCII bytes
	* If you need to transmit a string to system which you know can only
	* support 7bit ASCII, you could use this function.
	* @param string
	* @return string with non ASCII bytes removed
	* @package utf8
	* @see utf8_strip_non_ascii_ctrl
	*/
	public static function utf8_strip_non_ascii($str) {
		ob_start();
		while ( preg_match(
			'/^([\x00-\x7F]+)|([^\x00-\x7F]+)/S',
				$str, $matches) ) {
			if ( !isset($matches[2]) ) {
				echo $matches[0];
			}
			$str = substr($str, strlen($matches[0]));
		}
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
	}
	
	/**
	* Strip out device control codes in the ASCII range
	* which are not permitted in XML. Note that this leaves
	* multi-byte characters untouched - it only removes device
	* control codes
	* @see http://hsivonen.iki.fi/producing-xml/#controlchar
	* @param string
	* @return string control codes removed
	*/
	public static function utf8_strip_ascii_ctrl($str) {
		ob_start();
		while ( preg_match(
			'/^([^\x00-\x08\x0B\x0C\x0E-\x1F\x7F]+)|([\x00-\x08\x0B\x0C\x0E-\x1F\x7F]+)/S',
				$str, $matches) ) {
			if ( !isset($matches[2]) ) {
				echo $matches[0];
			}
			$str = substr($str, strlen($matches[0]));
		}
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
	}
	
	/**
	* Strip out all non 7bit ASCII bytes and ASCII device control codes.
	* For a list of ASCII device control codes see the 2nd table here:
	* http://www.w3schools.com/tags/ref_ascii.asp
	*
	* @param string
	* @return boolean TRUE if it's all ASCII
	* @package utf8
	*/
	public static function utf8_strip_non_ascii_ctrl($str) {
		ob_start();
		while ( preg_match(
			'/^([\x09\x0A\x0D\x20-\x7E]+)|([^\x09\x0A\x0D\x20-\x7E]+)/S',
				$str, $matches) ) {
			if ( !isset($matches[2]) ) {
				echo $matches[0];
			}
			$str = substr($str, strlen($matches[0]));
		}
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
	}
	
	/**
	* Replace accented UTF-8 characters by unaccented ASCII-7 "equivalents".
	* The purpose of this function is to replace characters commonly found in Latin
	* alphabets with something more or less equivalent from the ASCII range. This can
	* be useful for converting a UTF-8 to something ready for a filename, for example.
	* Following the use of this function, you would probably also pass the string
	* through utf8_strip_non_ascii to clean out any other non-ASCII chars
	* Use the optional parameter to just deaccent lower ($case = -1) or upper ($case = 1)
	* letters. Default is to deaccent both cases ($case = 0)
	*
	* For a more complete implementation of transliteration, see the utf8_to_ascii package
	* available from the phputf8 project downloads:
	* http://prdownloads.sourceforge.net/phputf8
	*
	* @param string UTF-8 string
	* @param int (optional) -1 lowercase only, +1 uppercase only, 1 both cases
	* @param string UTF-8 with accented characters replaced by ASCII chars
	* @return string accented chars replaced with ascii equivalents
	* @author Andreas Gohr <andi@splitbrain.org>
	* @package utf8
	*/
	public static function utf8_accents_to_ascii( $str, $case=0 ){
	
		static $UTF8_LOWER_ACCENTS = NULL;
		static $UTF8_UPPER_ACCENTS = NULL;
	
		if($case <= 0){
	
			if ( is_null($UTF8_LOWER_ACCENTS) ) {
				$UTF8_LOWER_ACCENTS = array(
	  'à' => 'a', 'ô' => 'o', 'ď' => 'd', 'ḟ' => 'f', 'ë' => 'e', 'š' => 's', 'ơ' => 'o',
	  'ß' => 'ss', 'ă' => 'a', 'ř' => 'r', 'ț' => 't', 'ň' => 'n', 'ā' => 'a', 'ķ' => 'k',
	  'ŝ' => 's', 'ỳ' => 'y', 'ņ' => 'n', 'ĺ' => 'l', 'ħ' => 'h', 'ṗ' => 'p', 'ó' => 'o',
	  'ú' => 'u', 'ě' => 'e', 'é' => 'e', 'ç' => 'c', 'ẁ' => 'w', 'ċ' => 'c', 'õ' => 'o',
	  'ṡ' => 's', 'ø' => 'o', 'ģ' => 'g', 'ŧ' => 't', 'ș' => 's', 'ė' => 'e', 'ĉ' => 'c',
	  'ś' => 's', 'î' => 'i', 'ű' => 'u', 'ć' => 'c', 'ę' => 'e', 'ŵ' => 'w', 'ṫ' => 't',
	  'ū' => 'u', 'č' => 'c', 'ö' => 'oe', 'è' => 'e', 'ŷ' => 'y', 'ą' => 'a', 'ł' => 'l',
	  'ų' => 'u', 'ů' => 'u', 'ş' => 's', 'ğ' => 'g', 'ļ' => 'l', 'ƒ' => 'f', 'ž' => 'z',
	  'ẃ' => 'w', 'ḃ' => 'b', 'å' => 'a', 'ì' => 'i', 'ï' => 'i', 'ḋ' => 'd', 'ť' => 't',
	  'ŗ' => 'r', 'ä' => 'ae', 'í' => 'i', 'ŕ' => 'r', 'ê' => 'e', 'ü' => 'ue', 'ò' => 'o',
	  'ē' => 'e', 'ñ' => 'n', 'ń' => 'n', 'ĥ' => 'h', 'ĝ' => 'g', 'đ' => 'd', 'ĵ' => 'j',
	  'ÿ' => 'y', 'ũ' => 'u', 'ŭ' => 'u', 'ư' => 'u', 'ţ' => 't', 'ý' => 'y', 'ő' => 'o',
	  'â' => 'a', 'ľ' => 'l', 'ẅ' => 'w', 'ż' => 'z', 'ī' => 'i', 'ã' => 'a', 'ġ' => 'g',
	  'ṁ' => 'm', 'ō' => 'o', 'ĩ' => 'i', 'ù' => 'u', 'į' => 'i', 'ź' => 'z', 'á' => 'a',
	  'û' => 'u', 'þ' => 'th', 'ð' => 'dh', 'æ' => 'ae', 'µ' => 'u', 'ĕ' => 'e',
				);
			}
	
			$str = str_replace(
					array_keys($UTF8_LOWER_ACCENTS),
					array_values($UTF8_LOWER_ACCENTS),
					$str
				);
		}
	
		if($case >= 0){
			if ( is_null($UTF8_UPPER_ACCENTS) ) {
				$UTF8_UPPER_ACCENTS = array(
	  'À' => 'A', 'Ô' => 'O', 'Ď' => 'D', 'Ḟ' => 'F', 'Ë' => 'E', 'Š' => 'S', 'Ơ' => 'O',
	  'Ă' => 'A', 'Ř' => 'R', 'Ț' => 'T', 'Ň' => 'N', 'Ā' => 'A', 'Ķ' => 'K',
	  'Ŝ' => 'S', 'Ỳ' => 'Y', 'Ņ' => 'N', 'Ĺ' => 'L', 'Ħ' => 'H', 'Ṗ' => 'P', 'Ó' => 'O',
	  'Ú' => 'U', 'Ě' => 'E', 'É' => 'E', 'Ç' => 'C', 'Ẁ' => 'W', 'Ċ' => 'C', 'Õ' => 'O',
	  'Ṡ' => 'S', 'Ø' => 'O', 'Ģ' => 'G', 'Ŧ' => 'T', 'Ș' => 'S', 'Ė' => 'E', 'Ĉ' => 'C',
	  'Ś' => 'S', 'Î' => 'I', 'Ű' => 'U', 'Ć' => 'C', 'Ę' => 'E', 'Ŵ' => 'W', 'Ṫ' => 'T',
	  'Ū' => 'U', 'Č' => 'C', 'Ö' => 'Oe', 'È' => 'E', 'Ŷ' => 'Y', 'Ą' => 'A', 'Ł' => 'L',
	  'Ų' => 'U', 'Ů' => 'U', 'Ş' => 'S', 'Ğ' => 'G', 'Ļ' => 'L', 'Ƒ' => 'F', 'Ž' => 'Z',
	  'Ẃ' => 'W', 'Ḃ' => 'B', 'Å' => 'A', 'Ì' => 'I', 'Ï' => 'I', 'Ḋ' => 'D', 'Ť' => 'T',
	  'Ŗ' => 'R', 'Ä' => 'Ae', 'Í' => 'I', 'Ŕ' => 'R', 'Ê' => 'E', 'Ü' => 'Ue', 'Ò' => 'O',
	  'Ē' => 'E', 'Ñ' => 'N', 'Ń' => 'N', 'Ĥ' => 'H', 'Ĝ' => 'G', 'Đ' => 'D', 'Ĵ' => 'J',
	  'Ÿ' => 'Y', 'Ũ' => 'U', 'Ŭ' => 'U', 'Ư' => 'U', 'Ţ' => 'T', 'Ý' => 'Y', 'Ő' => 'O',
	  'Â' => 'A', 'Ľ' => 'L', 'Ẅ' => 'W', 'Ż' => 'Z', 'Ī' => 'I', 'Ã' => 'A', 'Ġ' => 'G',
	  'Ṁ' => 'M', 'Ō' => 'O', 'Ĩ' => 'I', 'Ù' => 'U', 'Į' => 'I', 'Ź' => 'Z', 'Á' => 'A',
	  'Û' => 'U', 'Þ' => 'Th', 'Ð' => 'Dh', 'Æ' => 'Ae', 'Ĕ' => 'E',
				);
			}
			$str = str_replace(
					array_keys($UTF8_UPPER_ACCENTS),
					array_values($UTF8_UPPER_ACCENTS),
					$str
				);
		}
	
		return $str;
	
	}
	
	/**
	* Tests a string as to whether it's valid UTF-8 and supported by the
	* Unicode standard
	* Note: this function has been modified to simple return true or false
	* @author <hsivonen@iki.fi>
	* @param string UTF-8 encoded string
	* @return boolean true if valid
	* @see http://hsivonen.iki.fi/php-utf8/
	* @see utf8_compliant
	* @package utf8
	*/
	public static function utf8_is_valid($str) {
	
		$mState = 0;     // cached expected number of octets after the current octet
						 // until the beginning of the next UTF8 character sequence
		$mUcs4  = 0;     // cached Unicode character
		$mBytes = 1;     // cached expected number of octets in the current sequence
	
		$len = strlen($str);
	
		for($i = 0; $i < $len; $i++) {
	
			$in = ord($str{$i});
	
			if ( $mState == 0) {
	
				// When mState is zero we expect either a US-ASCII character or a
				// multi-octet sequence.
				if (0 == (0x80 & ($in))) {
					// US-ASCII, pass straight through.
					$mBytes = 1;
	
				} else if (0xC0 == (0xE0 & ($in))) {
					// First octet of 2 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x1F) << 6;
					$mState = 1;
					$mBytes = 2;
	
				} else if (0xE0 == (0xF0 & ($in))) {
					// First octet of 3 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x0F) << 12;
					$mState = 2;
					$mBytes = 3;
	
				} else if (0xF0 == (0xF8 & ($in))) {
					// First octet of 4 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x07) << 18;
					$mState = 3;
					$mBytes = 4;
	
				} else if (0xF8 == (0xFC & ($in))) {
					/* First octet of 5 octet sequence.
					*
					* This is illegal because the encoded codepoint must be either
					* (a) not the shortest form or
					* (b) outside the Unicode range of 0-0x10FFFF.
					* Rather than trying to resynchronize, we will carry on until the end
					* of the sequence and let the later error handling code catch it.
					*/
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x03) << 24;
					$mState = 4;
					$mBytes = 5;
	
				} else if (0xFC == (0xFE & ($in))) {
					// First octet of 6 octet sequence, see comments for 5 octet sequence.
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 1) << 30;
					$mState = 5;
					$mBytes = 6;
	
				} else {
					/* Current octet is neither in the US-ASCII range nor a legal first
					 * octet of a multi-octet sequence.
					 */
					return FALSE;
	
				}
	
			} else {
	
				// When mState is non-zero, we expect a continuation of the multi-octet
				// sequence
				if (0x80 == (0xC0 & ($in))) {
	
					// Legal continuation.
					$shift = ($mState - 1) * 6;
					$tmp = $in;
					$tmp = ($tmp & 0x0000003F) << $shift;
					$mUcs4 |= $tmp;
	
					/**
					* End of the multi-octet sequence. mUcs4 now contains the final
					* Unicode codepoint to be output
					*/
					if (0 == --$mState) {
	
						/*
						* Check for illegal sequences and codepoints.
						*/
						// From Unicode 3.1, non-shortest form is illegal
						if (((2 == $mBytes) && ($mUcs4 < 0x0080)) ||
							((3 == $mBytes) && ($mUcs4 < 0x0800)) ||
							((4 == $mBytes) && ($mUcs4 < 0x10000)) ||
							(4 < $mBytes) ||
							// From Unicode 3.2, surrogate characters are illegal
							(($mUcs4 & 0xFFFFF800) == 0xD800) ||
							// Codepoints outside the Unicode range are illegal
							($mUcs4 > 0x10FFFF)) {
	
							return FALSE;
	
						}
	
						//initialize UTF8 cache
						$mState = 0;
						$mUcs4  = 0;
						$mBytes = 1;
					}
	
				} else {
					/**
					*((0xC0 & (*in) != 0x80) && (mState != 0))
					* Incomplete multi-octet sequence.
					*/
	
					return FALSE;
				}
			}
		}
		return TRUE;
	}
	
	/**
	* Tests whether a string complies as UTF-8. This will be much
	* faster than utf8_is_valid but will pass five and six octet
	* UTF-8 sequences, which are not supported by Unicode and
	* so cannot be displayed correctly in a browser. In other words
	* it is not as strict as utf8_is_valid but it's faster. If you use
	* is to validate user input, you place yourself at the risk that
	* attackers will be able to inject 5 and 6 byte sequences (which
	* may or may not be a significant risk, depending on what you are
	* are doing)
	* @see utf8_is_valid
	* @see http://www.php.net/manual/en/reference.pcre.pattern.modifiers.php#54805
	* @param string UTF-8 string to check
	* @return boolean TRUE if string is valid UTF-8
	* @package utf8
	*/
	public static function utf8_compliant($str) {
		if ( strlen($str) == 0 ) {
			return TRUE;
		}
		// If even just the first character can be matched, when the /u
		// modifier is used, then it's valid UTF-8. If the UTF-8 is somehow
		// invalid, nothing at all will match, even if the string contains
		// some valid sequences
		return (preg_match('/^.{1}/us',$str,$ar) == 1);
	}
	
	/**
	* Takes an UTF-8 string and returns an array of ints representing the
	* Unicode characters. Astral planes are supported ie. the ints in the
	* output can be > 0xFFFF. Occurrances of the BOM are ignored. Surrogates
	* are not allowed.
	* Returns false if the input string isn't a valid UTF-8 octet sequence
	* and raises a PHP error at level E_USER_WARNING
	* Note: this function has been modified slightly in this library to
	* trigger errors on encountering bad bytes
	* @author <hsivonen@iki.fi>
	* @param string UTF-8 encoded string
	* @return mixed array of unicode code points or FALSE if UTF-8 invalid
	* @see utf8_from_unicode
	* @see http://hsivonen.iki.fi/php-utf8/
	* @package utf8
	*/
	public static function utf8_to_unicode($str) {
		$mState = 0;     // cached expected number of octets after the current octet
						 // until the beginning of the next UTF8 character sequence
		$mUcs4  = 0;     // cached Unicode character
		$mBytes = 1;     // cached expected number of octets in the current sequence
	
		$out = array();
	
		$len = strlen($str);
	
		for($i = 0; $i < $len; $i++) {
	
			$in = ord($str{$i});
	
			if ( $mState == 0) {
	
				// When mState is zero we expect either a US-ASCII character or a
				// multi-octet sequence.
				if (0 == (0x80 & ($in))) {
					// US-ASCII, pass straight through.
					$out[] = $in;
					$mBytes = 1;
	
				} else if (0xC0 == (0xE0 & ($in))) {
					// First octet of 2 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x1F) << 6;
					$mState = 1;
					$mBytes = 2;
	
				} else if (0xE0 == (0xF0 & ($in))) {
					// First octet of 3 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x0F) << 12;
					$mState = 2;
					$mBytes = 3;
	
				} else if (0xF0 == (0xF8 & ($in))) {
					// First octet of 4 octet sequence
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x07) << 18;
					$mState = 3;
					$mBytes = 4;
	
				} else if (0xF8 == (0xFC & ($in))) {
					/* First octet of 5 octet sequence.
					*
					* This is illegal because the encoded codepoint must be either
					* (a) not the shortest form or
					* (b) outside the Unicode range of 0-0x10FFFF.
					* Rather than trying to resynchronize, we will carry on until the end
					* of the sequence and let the later error handling code catch it.
					*/
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 0x03) << 24;
					$mState = 4;
					$mBytes = 5;
	
				} else if (0xFC == (0xFE & ($in))) {
					// First octet of 6 octet sequence, see comments for 5 octet sequence.
					$mUcs4 = ($in);
					$mUcs4 = ($mUcs4 & 1) << 30;
					$mState = 5;
					$mBytes = 6;
	
				} else {
					/* Current octet is neither in the US-ASCII range nor a legal first
					 * octet of a multi-octet sequence.
					 */
					trigger_error(
							'utf8_to_unicode: Illegal sequence identifier '.
								'in UTF-8 at byte '.$i,
							E_USER_WARNING
						);
					return FALSE;
	
				}
	
			} else {
	
				// When mState is non-zero, we expect a continuation of the multi-octet
				// sequence
				if (0x80 == (0xC0 & ($in))) {
	
					// Legal continuation.
					$shift = ($mState - 1) * 6;
					$tmp = $in;
					$tmp = ($tmp & 0x0000003F) << $shift;
					$mUcs4 |= $tmp;
	
					/**
					* End of the multi-octet sequence. mUcs4 now contains the final
					* Unicode codepoint to be output
					*/
					if (0 == --$mState) {
	
						/*
						* Check for illegal sequences and codepoints.
						*/
						// From Unicode 3.1, non-shortest form is illegal
						if (((2 == $mBytes) && ($mUcs4 < 0x0080)) ||
							((3 == $mBytes) && ($mUcs4 < 0x0800)) ||
							((4 == $mBytes) && ($mUcs4 < 0x10000)) ||
							(4 < $mBytes) ||
							// From Unicode 3.2, surrogate characters are illegal
							(($mUcs4 & 0xFFFFF800) == 0xD800) ||
							// Codepoints outside the Unicode range are illegal
							($mUcs4 > 0x10FFFF)) {
	
							trigger_error(
									'utf8_to_unicode: Illegal sequence or codepoint '.
										'in UTF-8 at byte '.$i,
									E_USER_WARNING
								);
	
							return FALSE;
	
						}
	
						if (0xFEFF != $mUcs4) {
							// BOM is legal but we don't want to output it
							$out[] = $mUcs4;
						}
	
						//initialize UTF8 cache
						$mState = 0;
						$mUcs4  = 0;
						$mBytes = 1;
					}
	
				} else {
					/**
					*((0xC0 & (*in) != 0x80) && (mState != 0))
					* Incomplete multi-octet sequence.
					*/
					trigger_error(
							'utf8_to_unicode: Incomplete multi-octet '.
							'   sequence in UTF-8 at byte '.$i,
							E_USER_WARNING
						);
	
					return FALSE;
				}
			}
		}
		return $out;
	}
	
	/**
	* Takes an array of ints representing the Unicode characters and returns
	* a UTF-8 string. Astral planes are supported ie. the ints in the
	* input can be > 0xFFFF. Occurrances of the BOM are ignored. Surrogates
	* are not allowed.
	* Returns false if the input array contains ints that represent
	* surrogates or are outside the Unicode range
	* and raises a PHP error at level E_USER_WARNING
	* Note: this function has been modified slightly in this library to use
	* output buffering to concatenate the UTF-8 string (faster) as well as
	* reference the array by it's keys
	* @param array of unicode code points representing a string
	* @return mixed UTF-8 string or FALSE if array contains invalid code points
	* @author <hsivonen@iki.fi>
	* @see utf8_to_unicode
	* @see http://hsivonen.iki.fi/php-utf8/
	* @package utf8
	*/
	public static function utf8_from_unicode($arr) {
		ob_start();
	
		foreach (array_keys($arr) as $k) {
	
			# ASCII range (including control chars)
			if ( ($arr[$k] >= 0) && ($arr[$k] <= 0x007f) ) {
	
				echo chr($arr[$k]);
	
			# 2 byte sequence
			} else if ($arr[$k] <= 0x07ff) {
	
				echo chr(0xc0 | ($arr[$k] >> 6));
				echo chr(0x80 | ($arr[$k] & 0x003f));
	
			# Byte order mark (skip)
			} else if($arr[$k] == 0xFEFF) {
	
				// nop -- zap the BOM
	
			# Test for illegal surrogates
			} else if ($arr[$k] >= 0xD800 && $arr[$k] <= 0xDFFF) {
	
				// found a surrogate
				trigger_error(
					'utf8_from_unicode: Illegal surrogate '.
						'at index: '.$k.', value: '.$arr[$k],
					E_USER_WARNING
					);
	
				return FALSE;
	
			# 3 byte sequence
			} else if ($arr[$k] <= 0xffff) {
	
				echo chr(0xe0 | ($arr[$k] >> 12));
				echo chr(0x80 | (($arr[$k] >> 6) & 0x003f));
				echo chr(0x80 | ($arr[$k] & 0x003f));
	
			# 4 byte sequence
			} else if ($arr[$k] <= 0x10ffff) {
	
				echo chr(0xf0 | ($arr[$k] >> 18));
				echo chr(0x80 | (($arr[$k] >> 12) & 0x3f));
				echo chr(0x80 | (($arr[$k] >> 6) & 0x3f));
				echo chr(0x80 | ($arr[$k] & 0x3f));
	
			} else {
	
				trigger_error(
					'utf8_from_unicode: Codepoint out of Unicode range '.
						'at index: '.$k.', value: '.$arr[$k],
					E_USER_WARNING
					);
	
				// out of range
				return FALSE;
			}
		}
	
		$result = ob_get_contents();
		ob_end_clean();
		return $result;
	}	
}

interface iBStringUtf8 {
    public static function utf8_strlen($str);
	public static function utf8_strpos($str, $needle, $offset = NULL);
	public static function utf8_strrpos($str, $needle, $offset = NULL);
	public static function utf8_substr($str, $offset, $length = NULL);
	public static function utf8_strtolower($string);
	public static function utf8_strtoupper($string);
}