<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Base View class
 *
 * @package Framework.Classes
 */
abstract class BView extends BObject {
	
	/**
	 * The name of the view
	 *
	 * @var    array
	 */
	public $app;
	
	/**
	 * The name of the view
	 *
	 * @var    array
	 */
	public $renderer;
	
	/**
	 * The name of the view
	 *
	 * @var    array
	 */
	protected $_name = null;

	/**
	 * Registered models
	 *
	 * @var    array
	 */
	protected $_models = array();

	/**
	 * The default model
	 *
	 * @var	string
	 */
	protected $_defaultModel = null;

	/**
	 * Constructor
	 *
	 * @param   array  $config  A named configuration array for object construction.<br />
	 *                          name: the name (optional) of the view (defaults to the view class name suffix).<br />
	 *                          charset: the character set to use for display<br />
	 *                          escape: the name (optional) of the function to use for escaping strings<br />
	 *                          base_path: the parent path (optional) of the views directory (defaults to the component folder)<br />
	 *                          template_plath: the path (optional) of the layout directory (defaults to base_path + /views/ + view name<br />
	 *                          helper_path: the path (optional) of the helper files (defaults to base_path + /helpers/)<br />
	 *                          layout: the layout (optional) to use to display the view<br />
	 *
	 * @since   12.2
	 */
	public function __construct($config = array())
	{
		parent::__construct();
		
		// get renderer
		$this->app		= Bookzo::getInstance();
		$this->renderer = $this->app->renderer->view();
		
		// Set the view name
		if (empty($this->_name))
		{
			$this->_name = $this->getName();
		}
	}


	/**
	 * Method to get data from a registered model or a property of the view
	 *
	 * @param   string  $property  The name of the method to call on the model or the property to get
	 * @param   string  $default   The name of the model to reference or the default value [optional]
	 *
	 * @return  mixed  The return value of the method
	 *
	 * @since   12.2
	 */
	public function get($property, $default = null)
	{
		// If $model is null we use the default model
		if (is_null($default))
		{
			$model = $this->_defaultModel;
		}
		else
		{
			$model = strtolower($default);
		}

		// First check to make sure the model requested exists
		if (isset($this->_models[$model]))
		{
			// Model exists, let's build the method name
			$method = 'get' . ucfirst($property);

			// Does the method exist?
			if (method_exists($this->_models[$model], $method))
			{
				// The method exists, let's call it and return what we get
				$result = $this->_models[$model]->$method();

				return $result;
			}
		}

		// Degrade to BObject::get
		$result = parent::get($property, $default);

		return $result;
	}
	
	/**
	 * Method to get the view name
	 *
	 * The model name by default parsed using the classname, or it can be set
	 * by passing a $config['name'] in the class constructor
	 *
	 * @return  string  The name of the model
	 *
	 * @since   12.2
	 * @throws  Exception
	 */
	public function getName()
	{
		if (empty($this->_name))
		{
			$r = null;

			if (!preg_match('/View(.*)/i', get_class($this), $r))
			{
				throw new BViewException('JLIB_APPLICATION_ERROR_MODEL_GET_NAME', 500);
			}

			$this->_name = strtolower($r[1]);
		}

		return $this->_name;
	}
	
	/**
	 * Method to get the model object
	 *
	 * @param   string  $name  The name of the model (optional)
	 *
	 * @return  mixed  JModelLegacy object
	 *
	 * @since   12.2
	 */
	public function getModel($name = null)
	{
		if ($name === null)
		{
			$name = $this->_defaultModel;
		}

		return $this->_models[strtolower($name)];
	}

	/**
	 * Method to add a model to the view.  We support a multiple model single
	 * view system by which models are referenced by classname.  A caveat to the
	 * classname referencing is that any classname prepended by JModel will be
	 * referenced by the name without JModel, eg. JModelCategory is just
	 * Category.
	 *
	 * @param   JModelLegacy  $model    The model to add to the view.
	 * @param   boolean       $default  Is this the default model?
	 *
	 * @return  object   The added model.
	 *
	 * @since   12.2
	 */
	public function setModel($model, $default = false)
	{
		$name = strtolower($model->getName());
		$this->_models[$name] = $model;

		if ($default)
		{
			$this->_defaultModel = $name;
		}

		return $model;
	}
}

/**
 * Dedicated Exception for the AppView class
 *
 * @see AppView
 */
class BViewException extends BzException {}