<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

/**
 * Bz data class.
 *
 * This class acts as parent for all data classes
 * It adds default functions like a magic method call 
 * for all child classes
 *
 * @subpackage 	framework.classes
 * @since 		1.0.0
 */
 
## NOTE - take a good look at this class
// Also we should take care of same query and fetching twice. This is unneccessary
abstract class BookzoApiRequest {

	/**
	 * Reference to the global App object
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */
	public 	  $app;
	
	/**
	 * The result fetched by the query
	 *
	 * @var 	Mixed
	 * @since 	1.0.0
	 */
	protected $result;
	
	/**
	 * The raw data fetched by the query
	 *
	 * @var 	Mixed
	 * @since 	1.0.0
	 */
	protected $data;
	
	/**
	 * The current name of the class
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $name;
	
	/**
	 * The bookzo api url
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $url;
	
	/**
	 * Credentials for connecting to api
	 *
	 * @var 	array
	 * @since 	1.0.0
	 */
	protected $credentials = false;
	
	/**
	 * The query to execute 
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $query;
	
	/**
	 * The query last executed 
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $lastquery;
		
	/**
	 * Verify to peer 
	 *
	 * @var 	string
	 * @since 	1.0.0
	 */
	protected $verifypeer  	= false;
	
	/**
	 * Verify host 
	 *
	 * @var 	boolean
	 * @since 	1.0.0
	 */
	protected $verifyhost	= false;
	
	
	/**
	 * Set server timeout 
	 *
	 * @var 	boolean
	 * @since 	1.0.0
	 */
	protected $timeout		= 15;
	
	/**
	 * An array holding the request instances
	 *
	 * @var 	boolean
	 * @since 	1.0.0
	 */
	protected static $instances	= array();

	/**
	 * Class constructor 
	 *
	 * @param   B    $class  A reference to the BookzoApp
	 * @param   string   $name   A name to set if child class not found by helper
	 * @return  void
	 * @since   1.0.0
	 */
	public function __construct() 
	{
		// set some basic stuff
		$this->app 		= Bookzo::getInstance();
				
		
		$url			= $this->app->config->get('general.bookzourl', 'http://mijnreservering.info/api/');
		$account		= $this->app->config->get('general.accountname');
		$password		= $this->app->config->get('general.password');
		
		## NOTE - Error here if not found
		$this->url 			= $url;
		
		if ( strlen($account) && strlen($password) ) {
			$this->credentials 	= $account.':'.$password;
		}
	}
	
	/**
	 * Method to get a child class instance 
	 *
	 * @param   B    $class  A reference to the BookzoApp
	 * @param   string   $name   A name to set if child class not found by helper
	 * @return  void
	 * @since   1.0.0
	 */
	public static function getInstance($type = 'json')
	{
		if ( !isset(self::$instances[$type]) ) 
		{
			$name = 'BookzoApiRequest'. ucfirst($type);
			
			if ( !class_exists( $name ) ) {
				throw new BRuntimeException('Request type '. $type . ' could not be loaded');
			}
			
			self::$instances[$type] = new $name();
		}
		
		return self::$instances[$type];
	}
	
	/**
	 * Set the query for this request
	 * 
	 * @param 	BookzoQuery $query 	A bookzo query instance
	 * @return 	self 
	 * @access	public
	 * @since 	1.0.0
	 */
	public function setQuery( $query )
	{
		// query should be of type BookzoQuery
		if ( !( $query instanceof BookzoQuery) ) 
		{
			$this->query = null;
		}
		
		// we need a mehtod
		if ( !$query->getMethod() ) 
		{
			$this->query = null;
		}
		
		// add it to global
		$this->query = $query;
		
		// return for concatenating
		return $this;
	}
		
	/**
	 * Get the query for currentle active
	 * 
	 * @param 	none
	 * @return 	string	$query	the query set, if none the defaultquery 
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getQuery( $new = false )
	{
		if ( $new || is_null($this->query) ) 
		{
			return new BookzoQuery();
		}
		
		return $this->query;
	}
	
	public function cache( $name )
	{
		if ( !$name ) {
			$this->cachename = null;
		}
		
		$this->cachename = $cache;
	}
	
	public function setTimeout( $sec )
	{
		$this->timeout = (int) $sec ? $sec : 5;
		
		return $this;
	}
		
	/**
	 * Get the name for the current instance 
	 *
	 * @param	none
	 * @return  string	$name 	the current name
	 * @access	protected
	 * @since   1.0.0
	 */
	 
	## NOTE - Unify this function over all classes
	protected function getName()
	{
		if (empty($this->name))
		{
			$r = null;

			if (!preg_match('/ApiRequest(.*)/i', get_class($this), $r))
			{
				throw new BModelException('JLIB_APPLICATION_ERROR_MODEL_GET_NAME', 500);
			}

			$this->name = strtolower($r[1]);
		}

		return $this->name;
	}
			
	/**
	 * Get the raw data as fetched by the query
	 * 
	 * @param 	none
	 * @return 	mixed	$data	the data received and parsed.
	 * @access	protected
	 * @since 	1.0.0
	 */
	public function getData()
	{
		$this->execute();
		
		return $this->data;
	}
	
	protected function lowerkeys(&$array, $recursive = false)
	{
		$isobject = false;
		if ( is_object($array) ) {
			$array 		= (array) $array;
			$isobject 	= true;
		}
		
		$array = array_change_key_case($array);
		
		if ( $recursive ) 
		{
			foreach ($array as $key => $value) {
				if ( is_array($value) || is_object($value) ) 
				{
					self::lowerkeys($array[$key], true);
				}
			}
		}
		
		if ( $isobject ) {
			$array = (object) $array;
		}
	}
	
	/**
	 * Method to get the result excecuted by the last query
	 * Childclass should handle this one
	 * 
	 * @param 	none
	 * @return 	mixed
	 * @access	protected
	 * @since 	1.0.0
	 */
	abstract public function getResult($retry = true);
	
	/**
	 * Method to get a list of elements found
	 * Childclass should handle this one
	 * 
	 * @param 	none
	 * @return 	mixed
	 * @access	protected
	 * @since 	1.0.0
	 */
	abstract public function getResults($key = null, $retry = true, $returnfalse = false);
	
	/**
	 * Method to execute the query
	 * Childclass should handle this one
	 * 
	 * @param 	none
	 * @return 	mixed	true on succesfull query or RequestError otherwise.
	 * @access	protected
	 * @since 	1.0.0
	 */
	abstract public function execute( $retry = 0 );
}

/**
 * Dedicated Exception for the AppView class
 *
 * @see AppView
 */
class BookzoRequestException extends BRuntimeException {}