<?php
/**
 * @package   bookzo
 * @author    Invenda  http://www.invenda.nl
 * @copyright Copyright (C) Invenda
 * @license   http://www.gnu.org/licenses/gpl.html GNU/GPL
 */

/**
 * Base View class
 *
 * @package Framework.Classes
 */
class BookzoFavourites extends BObject {
	
	/**
	 * $desc 	Container of favourite items
	 * $var		array
	 */
	protected $items;
	
	/**
	 * $desc 	The cookie jar
	 * $var		object
	 */
	protected $jar;
	
	/**
	 * $desc 	The class identifier
	 * $var		string
	 */
	protected $identifier;
	
	/**
	 * $desc 	The cookie identifier
	 * $var		string
	 */
	protected $name = 'bookzofavourites';
	
	/**
     * Class constructor
     *
     * @return  object		$this 	Return object itself for chaining
     * @see     JForm
	 * @access 	protected
     * @since   3.1
     */
	public function __construct()
	{
		// constuct through parent
		parent::__construct();
				
		// open jar and get items
		$this->identifier 	= 'fav-'.uniqid();
		$this->jar	  		= $this->app->input->cookie;		
		$this->items 	 	= $this->getCurrent();
		
		return $this;
	}
	
	/**
     * Method to get the favourite items
     *
     * @param   
     * @return  array  		The items currently marked as favourite
	 * @access 	public
     * @since   3.1
     */
	public function getItems()
	{
		return array_values($this->items);
	}
	
	/**
     * Method to get the favourite class identifier
     *
     * @param   
     * @return  string	$identifier 	The class identifier
	 * @access 	public
     * @since   3.1
     */
	public function identifier()
	{
		return $this->identifier;
	}
	
	/**
     * Set the jar name.
     *
     * @param   mixed	$id		either an array or single id
     * @return  bool			true on succes
	 * @access 	public
     * @since   3.1
     */
	public function setName($name)
	{
		$this->name = (string) $name;
		
		return $this->store();
	}
	
	/**
     * Get the jar name
     *
     * @param   mixed	$id		either an array or single id
     * @return  bool			true on succes
	 * @access 	public
     * @since   3.1
     */
	public function getName()
	{
		return $this->name;
	}
	
	/**
     * Add an item to favourites
	 * We add it to the jar immediatly
     *
     * @param   mixed	$id		either an array or single id
     * @return  bool			true on succes
	 * @access 	public
     * @since   3.1
     */
	public function add($id)
	{
		// merge with existing
		$ids 		 = (array) $id;
		$this->items = array_unique(array_merge($this->items, $ids));
		
		return $this->store();
	}
	
	/**
     * Removes an item to favourites
	 * We add it to the jar immediatly
     *
     * @param   mixed	$id		either an array or single id
     * @return  bool			true on succes
	 * @access 	public
     * @since   3.1
     */
	public function remove($id)
	{
		// distract from existing
		$ids 		 = (array) $id;
		$this->items = array_diff($this->items, $ids);
		
		return $this->store();
	}
	
	/**
     * Get the nr of items
     *
     * @param   none
     * @return  bool			true on succes
	 * @access 	public
     * @since   3.1
     */
	public function getCount()
	{		
		return count($this->items);
	}
	
	/**
     * Get the nr of items
     *
     * @param   none
     * @return  bool			true on succes
	 * @access 	public
     * @since   3.1
     */
	public function isFavourite($id)
	{
		return in_array( (int) $id, $this->items );
	}
	
	public function button($id, $container)
	{		
		$options  	 	= array();
		$options['id'] 	= $id;
				
		if ( $this->app->input->get('layout') == 'favourites' && $container ) 
		{
			$options['parent'] = (string) $container;
		}
		
		$data 	= 'data-bz-favourite=\''. json_encode($options) .'\'';
		$isfav 	= $this->isFavourite($id);
		
		return $this->app->renderer->partial()->render('favbutton', compact('data', 'isfav'));
	}
	
	
	/**
     * Method to get the jar content
	 * We split the content to get the villa id's
     *
     * @param   
     * @return  array	$items	an array if ids
	 * @access 	public
     * @since   3.1
     */
	private function getCurrent()
	{
		$items 	= array();
		$injar 	= $this->jar->get($this->name, null);

		if ( $injar && strlen($injar) ) {
			$items = explode('|', base64_decode($injar));
			$this->app->array->toInteger($items);
		}
		
		return $items;
	}
	
	/**
     * Method to add an item to the jar
	 * We set the experation time at one year
	 * We make a string out of an array for better browser support
     *
     * @param   
     * @return  bool	true on succes
	 * @access 	public
     * @since   3.1
     */
	 ## NOTE - We might want to add config for exp. date
	private function store()
	{
		$expire  = time() + (60 * 60 * 24 * 365);
		$store   = base64_encode(implode('|', $this->items));
			
		$this->jar->set($this->name, $store, $expire, '/');
		
		return true;
	}
}