<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

/**
 * Bz data class.
 *
 * This class acts as parent for all data classes
 * It adds default functions like a magic method call 
 * for all child classes
 *
 * @subpackage 	framework.classes
 * @since 		1.0.0
 */
class BookzoFilters extends BObject {
	
	/**
	 * Holder for all filters
	 *
	 * @var 	array
	 * @since 	1.0.0
	 */	
	protected $filters;
	
	/**
	 * Holder for all possible filters according to added objects
	 *
	 * @var 	array
	 * @since 	1.0.0
	 */	
	protected $possiblefilters;	
	
	/**
	 * Holder for all active filters (clicked values)
	 *
	 * @var 	array
	 * @since 	1.0.0
	 */	
	protected $activefilters;
	
	/**
	 * Holder for all added queried objects
	 *
	 * @var 	array
	 * @since 	1.0.0
	 */	
	protected $objects 	= array();
	
	/**
	 * Holder for all filtered queried objects
	 *
	 * @var 	mixed	array/null
	 * @since 	1.0.0
	 */	
	protected $filtered;	
	
	/**
	 * Holder for all filters joined into groups
	 *
	 * @var 	App
	 * @since 	1.0.0
	 */	
	protected $groups;	
	
	/* 
	 * Get filters
	 *
	 * @param 	void
	 * @return	array	an array of filters
	 * @since	1.0.0
	 */
	public function getFilters( $name = null )
	{
		// get filters just once
		if ( is_null( $this->filters ) ) {
			$this->filters = $this->app->api->getFilters();
		}
		
		// return by array search
		return $this->app->array->returnMixed($this->filters, $name);
	}
	
	/* 
	 * Get filters grouped by name
	 *
	 * @param 	void
	 * @return	array	an array of filters categorized by group
	 * @since	1.0.0
	 */
	public function getGroups( $name = null )
	{
		// do it just once
		if ( is_null( $this->groups ) )
		{
			$groups 	= array();
			$filters 	= $this->getFilters();
			
			// defide filters into groups
			foreach ( $filters as $filter ) 
			{
				$groups[ $filter->get('groupname') ][] = $filter;
			}
			
			$this->groups = $groups;
		}
		
		// return by array search
		return $this->app->array->returnMixed($this->groups, $name);
	}
	
	/* 
	 * Get filters grouped by name and referenced against all found objects
	 * This is used in the widget if no objects are set we return nothing.
	 *
	 * @param 	void
	 * @return	array	an array of filters categorized by group
	 * @since	1.0.0
	 */
	public function getPossibleFilters( $showempty = false )
	{
		// do it just once
		if ( is_null( $this->possiblefilters ) )
		{
			$possiblefilters 	= array();
			$filters 			= $this->getFilters();
			$return 			= array();
			
			if ( count($this->objects) ) 
			{
				// defide filters into groups
				foreach ( $filters as $filter ) 
				{
					$filter->bindObjects($this->objects, $this->getFiltered());
					$options = $filter->getOptions($showempty);
				
					if ( $showempty || count($options) ) {
						$possiblefilters[$filter->id] = $filter;
						//$possiblefilters[ $filter->get('groupname') ][] = $filter;
					}
				}
			}
			
			// order
			if ( count($possiblefilters) ) {
				uasort($possiblefilters, function($a, $b) { 				
					return ($a->sequence > $b->sequence); 
				});
				
				$grouped = array();
				foreach ( $possiblefilters as $k => $v ) {
					$grouped[$v->get('groupname')][] = $v;
				}
				$possiblefilters = $grouped;
			}
			
			
			
			$this->possiblefilters = $possiblefilters;
		}
		
		// return by array search
		return $this->possiblefilters;
	}
	
	/* 
	 * Add a object to the filter list.
	 *
	 * @param 	$object	mixed	Can be an array, int or bookzoobject
	 *
	 * @return	self
	 * @since	1.0.0
	 */
	public function addObject($object)
	{
		// loop if they want to add more then 1
		if ( is_array($object) ) {
			foreach ( $object as $o ) {
				$this->addObject($o);
			}
			
			return $this;
		}
		
		// if integer fetch bookzo object
		if ( is_int($object) )
		{
			$object = $this->app->api->getObject($object);
		}
		
		// add to stack if found and reset filtered and possible filters
		if ( $object instanceof BookzoObject && !array_key_exists($object->get('id'), $this->objects) )
		{
			$this->objects[$object->get('id')] = $object;
			$this->filtered 		= null;
			$this->possiblefilters  = null;
		}
		
		// return	
		return $this;
	}
	
	/* 
	 * Get filtered objects.
	 * Here we check the added object against each active filter value
	 *
	 * @param 	$object	mixed	Can be an array, int or bookzoobject
	 *
	 * @return	self
	 * @since	1.0.0
	 */
	public function getFiltered()
	{
		// do it just once
		if ( !is_array($this->filtered) ) 
		{
			// by default all objects are ok
			$this->filtered = $this->objects;
			
			// are there any active filters
			$activefilters	= $this->getActive(true);
			
			if ( count($activefilters) ) 
			{
				// loop all and remove those that don't have all active filters
				foreach ( $this->filtered as $oid => $object )
				{
					$features = $object->getFeatures();

					foreach ( $activefilters as $fid => $options )
					{
						// compare all active filters to objectfeatures
						if ( !isset($features[$fid]) || !in_array($features[$fid], $options) ) {
							unset($this->filtered[$oid]);
							continue(2);
						}
					}
				}
			}
		}
		
		return $this->filtered;		
	}
	
	
	public function isActive($id, $value)
	{
		// get active filters
		$active = $this->getActive();
		
		// check to see if filter is active
		if ( isset($active[$id]) && is_array($active[$id]) && in_array($value, $active[$id]) ) {

			return true;
		}
				
		return false;
		
	}
	
	public function getHiddenInputs( $return = false )
	{
		$active = $this->getActive();
		$inputs = array();
		
		foreach ( $active as $k => $v ) {
			if ( is_array($v) ) {
				foreach ( $v as $a => $b ) {
					$inputs[] = '<input type="hidden" value="1" name="filters['.$k.']['.$a.']" />';
				}
			} elseif (is_string($v) || is_int($v) ) {
				$inputs[] = 'input type="hidden" value="1" name="filters['.$k.']['.$v.']" />';
			}
		}
		
		if ( $return ) {
			return $inputs;
		}
		else {
			echo implode("\n", $inputs);
		}
	}
	
	public function getActive($asvalue = false)
	{
		// get active filters sorted by layout
		if ( is_null( $this->activefilters ) ) 
		{
			$layout					= $this->app->input->get('layout', 'group');
			$group					= (int) $this->app->input->get('id', 0);
			$layout					= $layout == 'group' ? 'group'.$group : $layout;
			$this->activefilters	= $this->app->adapter->sessionGet('bookzo.'.$layout.'.filters', array());
			
			if ( !is_array($this->activefilters) ) {
				$this->activefilters = array();
			}
		}
		
		// do we want keys or values returned
		if ( !$asvalue ) 
		{
			return $this->activefilters;
		}
		
		// values it is fetch them by filter
		$return = array();
		foreach ( $this->activefilters as $id => $selected ) 
		{
			$id 		 = (int) $id;
			$filter 	 = $this->getFilters($id);

			if ( $filter && is_array($selected) ) {
				foreach ( $selected as $k => $index )
				{
					$return[$id][$k] = $filter->getOptionValue($index);
				}
			}
			
		}
		
		return $return;		
		
	}	
}