<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzHelperApi extends BHelper {

	public $request;
	
	public $query;
	
	public $generalcachetime;
	
	public $periodscachetime;
	
	public function __construct($app)
	{
		parent::__construct($app);
		
		$this->request 			= BookzoApiRequest::getInstance();
		$this->query 			= $this->request->getQuery(true);
		$this->generalcachetime = (int) $this->app->config->get('general.general_cache', 15);
		$this->periodscachetime = (int) $this->app->config->get('general.periods_cache', 1);
	}
	
	/**
	 * Method to get a single object
	 * 
	 * @param 	id		int		an array of groups to return
	 * @return 	return	mixed	a BookzoGroup on succes or null otherwise
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getObjectsByGroup( $groupid )
	{
		$objects = $this->getObjects();
		$return  = array();
		
		if ( !( (int) $groupid ) ) {
			return $objects;
		}

		foreach ( $objects as $k => $object ) 
		{
			if ( $object->get('groupid') == $groupid ) {
				$return[$k] = $object;
			}
		}
		
		return $return;
	}
	
	public function getBookablePeriods($vars)
	{
		// build correct variables
		$vars   = $this->app->registry->create( (array) $vars );
		$query	= $this->app->registry->create();
		
		$query->set('From', 	$this->app->date->create($vars->get('from'))->format('Y-m-d'));
		$query->set('Until', 	$this->app->date->create($vars->get('until'))->format('Y-m-d'));
		$query->set('Group',	$vars->get('group', 0));
		$query->set('Persons',	$vars->get('persons', 0));
		$query->set('Pets',		$vars->get('pets', 0));
		
		// clean query and create key
		$unique	= $query->clean(true)->createkey();
		
		// do it just once
		if ( !isset($this->cache['bookableperiods'][$unique]) ) 
		{
			$results 	= $this->getResults('GetBookablePeriods', $query, null, array(), true, $this->periodscachetime, 'periodqueries' );
			$results 	= $this->app->array->objectKeysToArray($results);
			
			$this->cache['bookableperiods'][$unique] = $results;
		}
		
		// return
		return $this->cache['bookableperiods'][$unique];
	}
	
	public function getPriceDefinitions( $ids = null )
	{
		if ( !isset($this->cache['pricedefinitions']) ) 
		{
			$query	= $this->app->registry->create();
			$query->set('From', 	$this->app->date->today()->format('Y-m-d'));
			$query->set('Until', 	$this->app->date->nextyear()->format('Y-12-31'));
			
			$results 		= $this->getResults('GetPriceDefinitions', $query, 'id', array(), true, null, 'queries' );
			$results 		= $this->app->array->objectKeysToArray($results);
			$definitions 	= array();
			
			foreach ( $results as $id => $result ) {
				$definitions[$id] = new BookzoResponseData($result);
			}
			
			// sort them by nrofnights
			uasort($definitions, function($a, $b) { return strcmp($a->numberofnights, $b->numberofnights); });

			$this->cache['pricedefinitions'] = $definitions;
		}
		
		return $this->app->array->returnMixed($this->cache['pricedefinitions'], $ids, true);
	}
	
	public function getExtraCosts( $ids = null )
	{
		if ( !isset($this->cache['extracosts']) ) 
		{
			$results 	= $this->getResults('GetExtraCosts', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);			
			$costs 		= array();
			
			foreach ( $results as $result ) 
			{
				if ( isset($result->id) ) {
					$costs[$result->id] = new BookzoResponseData($result);
				}
			}
			
			$this->cache['extracosts'] = $costs;
		}
		
		return $this->app->array->returnMixed($this->cache['extracosts'], $ids);	
	}
	
	public function getSettings()
	{
		if ( !isset($this->cache['settings']) ) 
		{		
			$result 		= $this->getResult('GetSettings');

			$this->cache['settings'] = $result;
		}
		
		return $this->cache['settings'];
	}
	
	public function getAvailableMonths($group = 0, $persons = 0, $pets = 0 )
	{
		if ( !isset($this->cache['availablemonths']) ) 
		{
			$vars			 	= array();
			$vars['GroupId']	= (int) $group;
			$vars['Persons']	= (int) $persons;
			$vars['Pets']		= (int) $pets;
			
			$this->app->array->clearArray($vars);
			
			$results 	= $this->getResults('GetBookableMonths', $vars, null, array(), true, $this->periodscachetime, 'periodqueries' );

			## NOTE - For now bookzo does not return sorted results
			sort($results);

			$this->cache['availablemonths'] = $results;
		}
		
		return $this->cache['availablemonths'];
	}

	/**
	 * Method to get a list of object from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoObject objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getFreeObjects($arrival, $departure, $group = 0, $persons = 0, $pets = 0 ) 
	{
		// get free objects just once
		## NOTE - We should cache according to vars
		if ( !isset($this->cache['freeobjects']) ) 
		{
			$vars			 			= array();
			$vars['Arrival'] 			= $this->app->date->create($arrival)->format('Y-m-d');
			$vars['Departure'] 			= $this->app->date->create($departure)->format('Y-m-d');
			$vars['GroupId']			= (int) $group;
			$vars['NumberOfPersons']	= (int) $persons;
			$vars['NumberOfPets']		= (int) $pets;
			
			$this->app->array->clearArray($vars);
			
			$results 	= $this->getResults('GetFreeObjects', $vars, 'objectid', array(), true, $this->periodscachetime, 'periodqueries' );
			$results 	= $this->app->array->objectKeysToArray($results);
			$return		= array();
			
			// modify in correct format
			foreach ( $results as $result ) {
				$tmp 			= new stdClass();
				$tmp->objectid 	= $result->objectid;
				$tmp->price		= $this->app->price->create( $result, array('arrival' => $vars['Arrival'], 'departure' => $vars['Departure'])); 
				
				$return[$result->objectid] = $tmp;
			}

			$this->cache['freeobjects'] = $return;
		}
		
		return $this->cache['freeobjects'];
	}

	
	/**
	 * Method to get a list of object from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoObject objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getObjects( $ids = null ) 
	{
		// get object just once
		if ( !isset($this->cache['objects']) ) 
		{		
			if ( !$results = $this->app->cache->get('objects', $this->generalcachetime ) ) 
			{				
				$this->app->adapter->sessionClear('bookzo.randomids');
			
				// get total number of accommodations
				$total		= $this->app->config->get('bookzo.numberofaccommodations', 0);
				$loops 		= ceil($total/10);
				$fetched	= array();			
				$results	= array();

				// loop until maxloops reached
				for ( $i = 0; $i < $loops; $i++ ) 
				{
					$fetched = $this->getResults( 'GetObjects', array('Page' => $i), null, array(), false, null, 'queries', true);
					
					if ( is_array($fetched) && count($fetched) )
					{
						$results = array_merge($results, $fetched);
					}
					// no need to proceed on error
					elseif ( $i != $loops ) {
						break;
					}
				}
				
				// get from cache
				$cachedresults 		  = $this->app->cache->get('objects', 0);
				$cachedresultsdecoded = json_decode($cachedresults);
				
				// if results doesn't match total we take te cache
				if ( count($results) == $total || count($results) > count($cachedresultsdecoded) )
				{
					$this->app->cache->cache('objects', json_encode($results));
				}
				else {
					//$results = $this->app->cache->get('objects', 0);
					$this->app->cache->cache('objects', $cachedresults);
					//$results = json_decode($results);
					$results = $this->app->array->objectKeysToArray($cachedresultsdecoded);
				}
			}
			else {

				$results = json_decode($results);
				$results = $this->app->array->objectKeysToArray($results);
			}

			$objects = array();
			
			foreach ( $results as $result ) 
			{
				if ( isset($result->id) ) {
					$objects[$result->id] = new BookzoObject($result);
				}
			}			

			$this->cache['objects'] = $objects;
		}
		
		return $this->app->array->returnMixed($this->cache['objects'], $ids);				
	}
	
	/**
	 * Method to get a list of groups from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoGroup objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getGroups( $ids = null )
	{
		// get groups just once
		if ( !isset($this->cache['groups']) ) 
		{
			$results 	= $this->getResults('GetGroups', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);			
			$groups 	= array();
			
			foreach ( $results as $result ) 
			{
				if ( isset($result->id) ) {
					$groups[$result->id] = new BookzoGroup($result);
				}
			}
			
			$this->cache['groups'] = $groups;
		}
		
		return $this->app->array->returnMixed($this->cache['groups'], $ids);		
	}
	
	/**
	 * Method to get a list of groups from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoGroup objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getCountries( $ids = null )
	{
		// get groups just once
		if ( !isset($this->cache['countries']) ) 
		{
			$results 	= $this->getResults('GetCountries', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);	
			$countries 	= array();
			
			foreach ( $results as $id => $result ) 
			{
				$countries[$id] = new BookzoResponseData($result);
			}
			
			$this->cache['countries'] = $countries;
		}
		
		return $this->app->array->returnMixed($this->cache['countries'], $ids);				
	}
	
	/**
	 * Method to get a list of languages from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoGroup objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getLanguages( $ids = null )
	{
		// get languages just once
		if ( !isset($this->cache['languages']) ) 
		{
			$results 	= $this->getResults('GetLanguages', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);	
			$languages 	= array();
			
			foreach ( $results as $id => $result ) 
			{
				$languages[$id] = new BookzoResponseData($result);
			}
			
			$this->cache['languages'] = $languages;
		}
		
		return $this->app->array->returnMixed($this->cache['languages'], $ids);		
	}
	
	/**
	 * Method to get a list of groups from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoGroup objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getFilters( $ids = null )
	{
		// get filters just once
		if ( !isset($this->cache['filters']) ) 
		{
			$results 	= $this->getResults('getFilterFeatures', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);
			$filters	= array();					
			$i 			= 0;
			
			foreach ( $results as $result ) 
			{
				$group	 	= new BookzoResponseData($result);
				$groupname = $group->translate('name');
				
				foreach ( $group->features as $filter ) {					
					if ( (int) $filter->useforfilter == 1 ) {
						$filters[$filter->id] = new BookzoFiltersFilter($filter);
						$filters[$filter->id]->set('groupname', $groupname);
						$i++;
					}
				}
			}
			
			$this->cache['filters'] = $filters;
		}
		
		return $this->app->array->returnMixed($this->cache['filters'], $ids);		
	}
	
	/**
	 * Method to get a list of groups from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoGroup objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getFeatures( $ids = null )
	{
		// get features just once
		if ( !isset($this->cache['features']) ) 
		{
			$results 	= $this->getResults('getFeatures', array(), 'id', array());	
			$results 	= $this->app->array->objectKeysToArray($results);			
			$list 		= array();			
			
			foreach ( $results as $result ) 
			{
				$groupname = $result->name;
				
				if ( isset($result->texts) && is_array($result->texts) ) {
					$lang 	= $this->app->language->getId();
					
					foreach ( $result->texts as $language ) 
					{
						if ( $language->languageid == $lang && isset($language->name) && !empty($language->name) ) {
							$groupname = $language->name;
							break;
						}
					}
				}
				
				foreach ( $result->features as $features ) {
					$id				   		= $features->id;
					$features->groupname 	= $groupname;
					
					$list[$id] = new BookzoFeaturesFeature($features);
				}
			}
			
			$this->cache['features'] = $list;
		}
		
		return $this->app->array->returnMixed($this->cache['features'], $ids);		
	}
	
	public function getActions( $active = false, $ids = null )
	{
		// get languages just once
		if ( !isset($this->cache['actions']) ) 
		{
			$results 	= $this->getResults('GetActions', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);	
			$actions 	= array();
			
			foreach ( $results as $id => $result ) 
			{
				$actions[$id] = new BookzoAction($result);
			}
			
			$this->cache['actions'] = $actions;
		}
		
		$actions = $this->cache['actions'];
		
		// do we want only active actions
		if ( $active && is_null($ids) )
		{
			$today 	 = $this->app->date->create()->format('Y-m-d');
			
			foreach ( $actions as $k => $action ) {
				$frombookable 	= $action->getDate('frombookingdate');
				$untilbookable 	= $action->getDate('untilbookingdate');
      			$untilarrival 	= $action->getDate('untilarrivaldate');
				
				if ( $today < $frombookable || $today > $untilbookable || $today > $untilarrival || $action->get('onlyforage') || $action->get('onlywithcouponcode') ) {
					unset($actions[$k]);
				}
			}
		}
		
		return $this->app->array->returnMixed($actions, $ids);			
	}
	
	public function getProspect($oid, $arrival, $departure, $persons = 2, $seniors = 0, $childs = 0, $babys = 0, $pets = 0, $items = array(), $coupon = null, $group = 0)
	{					
		// make unique string for caching
		## NOTE = also add in items
		$unique	  = base64_encode($oid.$arrival.$departure.$persons.$group);
		
		if ( !isset($this->cache['prospect'][$unique]) ) 
		{
			$result = false;
			
			if ( (int) $oid || (int) $group ) 
			{
				// build the vars
				$vars			 			= array();
				if ( $group ) {
					$vars['GroupId'] = $group;
				}
				else {
					$vars['ObjectId'] 			= (int) $oid;
				}
				$vars['Arrival'] 			= $this->app->date->create($arrival)->format('Y-m-d');
				$vars['Departure'] 			= $this->app->date->create($departure)->format('Y-m-d');
				$vars['Adults']				= (int) $persons;
				$vars['Seniors']			= (int) $seniors;
				$vars['Childs']				= (int) $childs;
				$vars['Babies']				= (int) $babys;
				$vars['Pets']				= (int) $pets;
				$vars['LanguageId']			= $this->app->language->getId();
				
				if ( strlen(trim($coupon)) ) {
					$vars['CouponCode'] = $coupon;
				}
							
				// do we add items
				$itemparts = array();
				foreach ( $items as $id => $amount )
				{
					if ( (int) $id && (int) $amount ) {
						$itemparts[] = (int) $id;
						$itemparts[] = (int) $amount;
					}
				}
				
				if ( count($itemparts) )
				{
					$vars['Costs'] = implode(',', $itemparts);
				}
				
				// try fetching the prospect	
				try {
					$this->query->clear();
					$this->query->set('GetReservationProspect');
					$this->query->wheres($vars);
	
					$result = $this->request->setQuery( $this->query )->getResult();
				}
				catch ( BookzoRequestException $e ) {
					$result	= false;
				}
			}
			
			// set it into the internal cache
			$prospect = new BookzoProspect($result);
			$prospect->set('id', $unique);
			
			$this->cache['prospect'][$unique] = $prospect;
		}
		
		return $this->cache['prospect'][$unique];
	}
	
	public function book($fields)
	{
		$return = false;		
		$this->query->clear();
		$this->query->set('SubmitReservation');

		foreach ( $fields as $k => $v ) 
		{
			if ( strlen(trim($v)) ) {
				$this->query->where($k, $v);
			}
		}

		try {
			## NOTE - Set timeout to higher level for now until fix
			$this->request->setTimeout(15);
			$result = $this->request->setQuery( $this->query )->getResult();

			if ( is_object($result) && isset($result->arrival) ) {
				$return = $result;
			}
		}
		catch ( BookzoRequestException $e ) {
			$return	= false;
		}
		
		## NOTE - Reset timeout for now
		$this->request->setTimeout(5);
		
		return $return;
	}
	
	/**
	 * Get a list of free periods with actions
	 *
	 * @access	public
	 * @since   1.0.0
	 *
	 * DEPRECATED since 1.2.0 use $bookzo->api->getPrices() instead
	 */
	public function getFreePeriodsWithAction($actions)
	{
		return $this->getPrices( compact('actions') );
	}
	
	/**
	 * Get a list of free periods
	 *
	 * @access	public
	 * @since   1.0.0
	 *
	 * DEPRECATED since 1.2.0 use $bookzo->api->getPrices() instead
	 */
	public function getFreePeriods($from, $until, $objectid = 0, $group = 0, $persons = 0, $pets = 0, $actions = 0)
	{
		return $this->getPrices( compact('from','until','objectid','group','persons','pets','actions') );
	}	
	
	public function getPrices($vars, $divide = false)
	{	
		// build correct variables
		$vars   = $this->app->registry->create( (array) $vars );
		$query	= $this->app->registry->create();
		$method = 'GetFreePeriods';
		
		$query->set('From', 	$this->app->date->create($vars->get('from', $this->app->date->today('Y-m-d')))->format('Y-m-d'));
		$query->set('Until', 	$this->app->date->create($vars->get('until', $this->app->date->nextyear('Y-m-d')))->format('Y-m-d'));
		$query->set('Persons',	$vars->get('persons', 0));
		$query->set('Pets',		$vars->get('pets', 0));
		
		// object or group
		if ( $objectid = $vars->get('objectid', 0) ) {
			$query->set('ObjectId', $objectid);
		}
		else if ( $groupid = $vars->get('group', 0) ) {
			$query->set('Group', $groupid );
		}

		// add actions
		if ( $actions = $vars->get('actions') ) {
			$query->set('Actions', is_array($actions) ? implode(',', $actions) : (int) $actions); 		
		}
		

		// make unique string for caching
		$unique	  = $query->clean(true)->createkey();

		// do it just once
		if ( !isset($this->cache['prices'][$unique]) ) 
		{
			$results 		= $this->getResults($method, $query, null, array(), true, $this->periodscachetime, 'periodqueries' );
			$return 		= array();
			
			foreach ( $results as $result ) {
				foreach ( $result->departures as $departure ) {
					$departure->arrival = $result->arrival;
					if ( $divide ) {
						$return[$departure->objectid][] = $departure;
					}
					else {
						$return[] = $departure;
					}
				}
			}

			// add to internal cache
			$this->cache['prices'][$unique] = $return;
		}
		
		// return
		return $this->cache['prices'][$unique];
	}	
	
	public function getArrivals($from, $until, $priceid = 0, $group = 0, $persons = 0, $pets = 0, $object = 0)
	{
		$today 	= $this->app->date->create()->format('Y-m-d');
		$from	= $this->app->date->create($from)->format('Y-m-d');
		$until 	= $this->app->date->create($until)->format('Y-m-d');
		$from	= $from < $today ? $today : $from;
		
		$vars				= array();
		$vars['From'] 		= $from;
		$vars['Until'] 		= $until;
		if ( (int) $object ) {
			$vars['Object'] 	= (int) $object;
			$method 			= 'GetPossibleArrivals';
		}
		else {
			$vars['Group']		= (int) $group;
			$vars['Persons']	= (int) $persons;
			$vars['Pets']		= (int) $pets;
			$method 			= 'GetPossibleArrivals';
		}
		
		## NOTE - We merge priceids but this is kinda silly
		## ASK Jon to add more then 1 priceid in request method
		$tmp		= array();
		$priceid	= explode('-', $priceid);
		foreach( $priceid as $a ) {
			$vars['PriceId'] = $a;
			
			$this->app->array->clearArray($vars);
			
			$results 	= $this->getResults($method, $vars, null, array(), true, $this->periodscachetime, 'periodqueries' );
			$arrivals  	= array();
			foreach ( $results as $a ) {
				$arrivals[] = $this->app->date->create($a)->format('Y-m-d');
			}
			
			$tmp = array_unique(array_merge($tmp, $arrivals));
			
		}
		
		## NOTE - For now bookzo does not return sorted results
		sort($tmp);
		
		return $tmp;
	}
	
	public function getDepartures($from, $group = 0, $persons = 0, $pets = 0, $object = 0)
	{
		$vars				= array();
		$vars['From'] 		= $from;
		
		if ( (int) $object ) {
			$vars['Object'] 	= (int) $object;
			$method 			= 'GetPossibleDepartures';
		}
		else {
			$vars['Group']		= (int) $group;
			$vars['Persons']	= (int) $persons;
			$vars['Pets']		= (int) $pets;
			$method 			= 'GetPossibleDepartures';
		}
		
		$this->app->array->clearArray($vars);
		
		// return if no vars
		if ( !count($vars) ) {
			return array();
		}
		
		$results 	= $this->getResults('GetPossibleDepartures', $vars, null, array(), true, $this->periodscachetime, 'periodqueries' );
		$departures = array();
		foreach ( $results as $a ) {
			$departures[] = $this->app->date->create($a)->format('Y-m-d');
		}
			
		## NOTE - For now bookzo does not return sorted results		
		sort($departures);
		
		return $departures;
	}
	
	public function getBooked($from, $until, $objectid = 0, $groupid = 0)
	{
		$today 	= $this->app->date->create()->format('Y-m-d');
		$from	= $this->app->date->create($from)->format('Y-m-d');
		$until 	= $this->app->date->create($until)->format('Y-m-d');
		//$from	= $from < $today ? $today : $from;
		
		$vars					= array();
		$vars['ObjectId']		= (int) $objectid;
		$vars['From']			= $from;
		$vars['Until']			= $until;
		$vars['ConcatPeriods'] 	= 'true';
		
		$this->app->array->clearArray($vars);
		
		$results 	= $this->getResults('GetBookedPeriods', $vars, null, array(), true, $this->periodscachetime, 'periodqueries' );
		$return 	= array();
		
		foreach ( $results as $result ) {
			$tmp				= new stdClass();
			$tmp->arrival 		= $this->app->date->create($result->arrival)->format('Y-m-d');
			$tmp->departure 	= $this->app->date->create($result->departure)->format('Y-m-d');
			$return[$tmp->arrival .'-'.$tmp->departure] = $tmp;
		}
		
		return $return;
	}
	
	/**
	 * Method to get a list of paymentmethods from bookzo
	 * 
	 * @param 	ids		array	an array of groups to return
	 * @return 	return	array	an array of BookzoGroup objects
	 * @access	public
	 * @since 	1.0.0
	 */
	public function getPaymentMethods( $ids = null )
	{
		// get languages just once
		if ( !isset($this->cache['paymentmethods']) ) 
		{
			$results 	= $this->getResults('Bookkeeping/GetPaymentMethods', array(), 'id', array());
			$results 	= $this->app->array->objectKeysToArray($results);	
			$methods 	= array();
			
			foreach ( $results as $id => $result ) 
			{
				$methods[$id] = new BookzoResponseData($result);
			}
			
			$this->cache['paymentmethods'] = $methods;
		}
		
		return $this->app->array->returnMixed($this->cache['paymentmethods'], $ids);		
	}
	
	public function addPayment( $trid, $amount, $desc, $eid = 0 )
	{
		$this->query->clear();
		$this->query->set('AddPaymentForTargetPay');
		
		$vars							= array();
		$vars['TargetpayTransactionId']	= $trid;
		$vars['PayDate']				= $this->app->date->create()->format('Y-m-d');
		$vars['Amount']					= number_format( (float) $amount, 2, '.', '');
		$vars['PaymentMethod']			= $this->app->config->get('bookzo.generatepaymentwhenpayedonline', 0) ? $this->app->config->get('pay.bookzopaymethod', 0) : 0;
		$vars['Description']			= $desc;
		$vars['ExternalId']				= $eid;
		
		$this->app->array->clearArray($vars, false);
		
		foreach ( $vars as $k => $v ) 
		{
			$this->query->where($k, $v);
		}
		
		try {
			echo (string) $this->query;
			$result = (bool) $this->request->setQuery( $this->query )->getResult( false );
		}
		catch ( BookzoRequestException $e ) {
			$return	= false;
		}
		
		return $return;
	}
	
	protected function getResults( $method, $vars = array(), $key = null, $default = false, $cache = true, $cachetime = null, $cachedir = 'queries', $returnfalse = false )
	{
		// vars should be an array
		if ( $vars instanceof BRegistry ) {
			$vars = $vars->toArray();
		}
		
		// set default values
		$found = false;
		$old   = $default;
				
		// do we load from cache
		if ( $cache ) 
		{
			$cachetime = $cachetime ? $cachetime : $this->generalcachetime;
			$cachename = strtolower($method);
			
			if ( is_array($vars) && count($vars) )
			{
				$cachename .= '.' . md5(http_build_query($vars));
			}
			
			// do we have a valid cache
			if ( $return = $this->app->cache->get($cachename, $cachetime, $cachedir) )
			{
				return json_decode($return);
			}
			else {
				// load old cache if api does not return anything. 
				// we can return old data to not get blank lists
				if ( $cached = $this->app->cache->get($cachename, 0, $cachedir) ) {
					$old = $cached;
					$old = json_decode($cached);
				}
			}
		}
		
		// build query
		$this->query->clear();
		$this->query->set($method);
		
		if ( count($vars) ) {
			$this->query->wheres($vars);
		}

		// try fetching results		
		try {					
			$return = $this->request->setQuery( $this->query )->getResults($key, false, $returnfalse);	
			
			if ( $return ) {
				$found	= true;	
			}
			else {
				if ( $returnfalse && $return === false ) {
					return false;
				}
				else {
					$return = $old;
				}
			}
		}
		catch ( BookzoRequestException $e ) {
			$return = $old;
		}
		
		// do we need to cache
		if ( $cache && $found ) 
		{
			$this->app->cache->cache( $cachename, json_encode($return), $cachedir );
		}
		
		// return
		return $return;
	}
	
	protected function getResult( $method, $vars = array(), $key = null, $default = false, $cache = true, $cachetime = null, $cachedir = 'queries' )
	{
		$return = $this->getResults($method, $vars, $key, $default, $cache, $cachetime, $cachedir);
		
		if ( !count($return) ) {
			return $default;
		}
		
		return is_array($return) ? reset($return) : null;
	}
}