<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Bz data helper class.
 *
 * Helper for managing and retrieving paths
 * We can register paths.
 * When loading files we search paths top to bottom
 *
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BzHelperArray extends BHelper {

	/**
	 * Function to convert array to integer values
	 *
	 * @param   array  &$array   The source array to convert
	 * @param   mixed  $default  A default value (int|array) to assign if $array is not an array
	 *
	 * @return  void
	 */
	public static function toInteger(&$array, $default = null, $null = true)
	{
		if (is_array($array))
		{
			foreach ($array as $i => $v)
			{
				if ( !$null && !(int) $v ) {
					continue;
				}
				
				$array[$i] = (int) $v;
			}
		}
		else
		{
			if ($default === null)
			{
				$array = array();
			}
			elseif (is_array($default))
			{
				self::toInteger($default, null);
				$array = $default;
			}
			else
			{
				$array = array((int) $default);
			}
		}
	}
	
	/**
	 * Function to clear array 
	 *
	 * @param   array  &$array   The source array to convert
	 * @param   mixed  $default  A default value (int|array) to assign if $array is not an array
	 *
	 * @return  void
	 */
	/**
     * Removes empty values from an array
     *
     * @access 	public
     * @param   $array 	array 	the array to clear
     * @return  array	
     * @since   1.0	  
     */
	public static function clearArray( &$array, $zero = true ) 
	{
		settype( $array, 'array' );
		
		foreach ( $array as $k => $v ) {
			if ( is_array($v) ) {
				$v = self::clearArray($v, $zero);
				
				if ( empty($v) ) {
					unset($array[$k]);
				}
				
				continue;
			}
			if ( !strlen(trim($v)) || is_null($v) || ($zero && empty($v)) ) {
				unset($array[$k]);
			}
		}
	}
	
	/**
	 * Utility function to map an array to a string.
	 *
	 * @param   array    $array         The array to map.
	 * @param   string   $inner_glue    The glue (optional, defaults to '=') between the key and the value.
	 * @param   string   $outer_glue    The glue (optional, defaults to ' ') between array elements.
	 * @param   boolean  $keepOuterKey  True if final key should be kept.
	 *
	 * @return  string   The string mapped from the given array
	 *
	 * @since   1.0
	 */
	public static function toString(array $array, $inner_glue = '=', $outer_glue = ' ', $keepOuterKey = false)
	{
		$output = array();

		foreach ($array as $key => $item)
		{
			if (is_array($item))
			{
				if ($keepOuterKey)
				{
					$output[] = $key;
				}

				// This is value is an array, go and do it again!
				$output[] = self::toString($item, $inner_glue, $outer_glue, $keepOuterKey);
			}
			else
			{
				$output[] = $key . $inner_glue . '"' . $item . '"';
			}
		}

		return implode($outer_glue, $output);
	}
		
	/**
	 * Utility function to return a value from a named array or a specified default
	 *
	 * @param   array   &$array   A named array
	 * @param   string  $name     The key to search for
	 * @param   mixed   $default  The default value to give if no key found
	 * @param   string  $type     Return type for the variable (INT, FLOAT, STRING, WORD, BOOLEAN, ARRAY)
	 *
	 * @return  mixed  The value from the source array
	 *
	 * @since   11.1
	 */
	public static function getValue(&$array, $name, $default = null, $type = '')
	{
		$result = null;

		if (isset($array[$name]))
		{
			$result = $array[$name];
		}

		// Handle the default case
		if (is_null($result))
		{
			$result = $default;
		}

		// Handle the type constraint
		switch (strtoupper($type))
		{
			case 'INT':
			case 'INTEGER':
				// Only use the first integer value
				@preg_match('/-?[0-9]+/', $result, $matches);
				$result = @(int) $matches[0];
				break;

			case 'FLOAT':
			case 'DOUBLE':
				// Only use the first floating point value
				@preg_match('/-?[0-9]+(\.[0-9]+)?/', $result, $matches);
				$result = @(float) $matches[0];
				break;

			case 'BOOL':
			case 'BOOLEAN':
				$result = (bool) $result;
				break;

			case 'ARRAY':
				if (!is_array($result))
				{
					$result = array($result);
				}
				break;

			case 'STRING':
				$result = (string) $result;
				break;

			case 'WORD':
				$result = (string) preg_replace('#\W#', '', $result);
				break;

			case 'NONE':
			default:
				// No casting necessary
				break;
		}
		return $result;
	}

	
	public static function inArray($value, $array, $case = false)
	{
		if ( $case ) {
			$array = array_map('strtolower', $array);
			$value = strtolower($value);
		}
		
		return in_array($value, $array);
	}
	
	public static function implodeRecursive($array, $glue = '', $filter = false)
	{
		if ( is_string($array) ) 
		{
			$string = $array;
		}
		else 
		{
			foreach ( $array as $k => $v ) {
				if ( is_array($v) ) {
					$array[$k] = self::implodeRecursive($v, $glue, $filter);
				}
			}
			
			if ( $filter ) {
				$array = array_filter($array, 'strlen');
			}
			
			$string = implode($glue, $array);
		}
		
		if ( $filter && empty($string) ) {
			return '';
		}
		
		return $string;
	}
	
	public static function lowerkeys(&$array, $recursive = false)
	{
		$isobject = false;
		if ( is_object($array) ) {
			$array 		= (array) $array;
			$isobject 	= true;
		}
		
		$array = array_change_key_case($array);
		
		if ( $recursive ) 
		{
			foreach ($array as $key => $value) {
				if ( is_array($value) || is_object($value) ) 
				{
					self::lowerkeys($array[$key], true);
				}
			}
		}
		
		if ( $isobject ) {
			$array = (object) $array;
		}
	}
	
	/**
	 * Faster then reset key
	 *
	 */
	public function firstkey($array)
	{
		foreach ( $array as $k => $v ) {
			return $k;
		}
		
		return null;
	}
	
	/**
	 * Keeps array in shape not like unshift
	 *
	 */
	public function first($array)
	{
		foreach ( $array as $k => $v ) {
			return $v;
		}
		
		return null;
	}
	
	/**
	 * Keeps array in shape not like pop
	 *
	 */
	public function last($array)
	{
		$tmp = array_reverse($array);
		
		return $this->first($tmp);
	}
	
	/**
	 * Method to determine if an array is an associative array.
	 *
	 * @param   array  $array  An array to test.
	 *
	 * @return  boolean  True if the array is an associative array.
	 *
	 * @since   1.0
	 */
	public static function isAssociative($array)
	{
		if (is_array($array))
		{
			foreach (array_keys($array) as $k => $v)
			{
				if ($k !== $v)
				{
					return true;
				}
			}
		}

		return false;
	}
	
	public static function natksort(&$array) {
    	$keys = array_keys($array);
    	natcasesort($keys);

    	foreach ($keys as $k) {
        	$new_array[$k] = $array[$k];
    	}

    	$array = $new_array;
    	return true;
	}
	
	public static function toUrl($array)
	{
		return http_build_query( (array) $array );
	}
	
	public static function extract( array $array, $key, $unique = false )
	{		
		$return = array();
		
		foreach ( $array as $k => $v ) {
			if ( $k === $key ) {
				$return[] = $v;
				continue;
			}
			elseif ( is_array($v) || is_object($v) ) {
				$tmp	= self::extract((array) $v, $key);
				if ( count($tmp) ) {
					$return = array_merge($return, $tmp);
					continue;
				}
			}
		}
		
		if ( $unique ) {
			$return = array_unique($return);
		}
		
		return $return;
	}
	
	public static function returnMixed( array $array, $key = null, $orderbyarray = false )
	{		
		// do we want more then one
		if ( is_array($key) && count($key) ) 
		{
			$return = array();
			
			if ( $orderbyarray ) {
				foreach ( $array as $k => $v ) {
					if ( in_array($k,$key) ) {
						$return[$k] = $array[$k];
					}
				}
			}
			else {
				foreach ( $key as $k ) {
					if ( isset($array[$k]) ) {
						$return[$k] = $array[$k];
					}
				}
			}
			
			return $return;
		}
		
		// or just one
		elseif ( !is_null($key) ) 
		{
			return isset($array[$key]) ? $array[$key] : false;
		}
		
		// or all	
		return $array;
	}
	
	/**
	 * Method to turn json_decode string back into array
	 * We do this because json_encode stores associative array into an object
	 *
	 * @param 	$array object|array the object or array to transdorm
	 * @return 	$return array	the returned data
	 * @access	public
	 */
	public static function objectKeysToArray($array)
	{
		$array 	= (array) $array;
		$return = array();
		
		foreach ( $array as $a => $b ) {
			if ( is_numeric($a) ) {
				$return[ (int) $a ] = $b;
			}
			else {
				$return[$a] = $b;
			}
		}
		
		return $return;
	}
}