<?php
/**
* @package   	Bz
* @author    	Invenda http://www.invenda.nl
* @copyright 	Copyright (C) Invenda.nl
* @license   	http://www.gnu.org/licenses/gpl.html GNU/GPL
*/

defined('_BOOKZO') or die;

/**
 * Helper for the loading of classes. 
 * Wrapper for JLoader
 * 
 * @subpackage 	framework.helpers
 * @since 		1.0.0
 */
class BLoader {

	/**
	 * The list of loaded data classes
	 * 
	 * @var array
	 * @since 1.0.0
	 */
	protected static $setup	  = false;
	
	/**
	 * The list of loaded data classes
	 * 
	 * @var array
	 * @since 1.0.0
	 */
	protected static $path	  = array();

	/**
	 * The list of loaded data classes
	 * 
	 * @var array
	 * @since 1.0.0
	 */
	protected static $classes = array();
	
	/**
	 * Container for registered library class prefixes and path lookups.
	 *
	 * @var    array
	 * @since  12.1
	 */
	protected static $prefixes = array();
	
	/**
	 * Directly register a class to the autoload list.
	 *
	 * @param   string   $class  The class name to register.
	 * @param   string   $path   Full path to the file that holds the class to register.
	 * @param   boolean  $force  True to overwrite the autoload path value for the class if it already exists.
	 *
	 * @return  void
	 *
	 * @since   11.1
	 */
	public static function register($class, $path, $force = true)
	{
		// Sanitize class name.
		$class = strtolower($class);

		// Only attempt to register the class if the name and file exist.
		if (!empty($class) && is_file($path))
		{
			// Register the class with the autoloader if not already registered or the force flag is set.
			if (empty(self::$classes[$class]) || $force)
			{
				self::$classes[$class] = $path;
			}
		}
	}
	
	/**
	 * Register a class prefix with lookup path.  This will allow developers to register library
	 * packages with different class prefixes to the system autoloader.  More than one lookup path
	 * may be registered for the same class prefix, but if this method is called with the reset flag
	 * set to true then any registered lookups for the given prefix will be overwritten with the current
	 * lookup path. When loaded, prefix paths are searched in a "last in, first out" order.
	 *
	 * @param   string   $prefix   The class prefix to register.
	 * @param   string   $path     Absolute file path to the library root where classes with the given prefix can be found.
	 * @param   boolean  $reset    True to reset the prefix with only the given lookup path.
	 * @param   boolean  $prepend  If true, push the path to the beginning of the prefix lookup paths array.
	 *
	 * @return  void
	 *
	 * @throws  RuntimeException
	 *
	 * @since   12.1
	 */
	public static function registerPrefix($prefix, $namespace, $reset = false, $append = false)
	{		
		// If the prefix is not yet registered or we have an explicit reset flag then set set the path.
		if (!isset(self::$prefixes[$prefix]) || $reset)
		{
			self::$prefixes[$prefix] = array($namespace);
		}
		// Otherwise we want to simply add the path to the prefix.
		else
		{
			if ($append)
			{
				self::$prefixes[$prefix][] = $namespace;
			}
			else
			{
				array_unshift(self::$prefixes[$prefix], $namespace);
			}
		}
	}
	
	/**
	 * Method to get the list of registered classes and their respective file paths for the autoloader.
	 *
	 * @return  array  The array of class => path values for the autoloader.
	 *
	 * @since   11.1
	 */
	public static function getClassList()
	{
		return self::$classes;
	}
	
	/**
	 * Method to get the list of registered classes and their respective file paths for the autoloader.
	 *
	 * @return  array  The array of class => path values for the autoloader.
	 *
	 * @since   11.1
	 */
	public static function getPrefixes()
	{
		return self::$prefixes;
	}
	
	/**
	 * Method to setup the autoloaders for the Joomla Platform.
	 * Since the SPL autoloaders are called in a queue we will add our explicit
	 * class-registration based loader first, then fall back on the autoloader based on conventions.
	 * This will allow people to register a class in a specific location and override platform libraries
	 * as was previously possible.
	 *
	 * @param   boolean  $enablePsr       True to enable autoloading based on PSR-0.
	 * @param   boolean  $enablePrefixes  True to enable prefix based class loading (needed to auto load the Joomla core).
	 * @param   boolean  $enableClasses   True to enable class map based class loading (needed to auto load the Joomla core).
	 *
	 * @return  void
	 *
	 * @since   12.3
	 */
	public static function setup(&$path, $enablePrefixes = true, $enableClasses = true)
	{
		// we need the path helper
		if ( !($path instanceof BPathHelper) ) {
			throw new BzRuntimeException('The path helper should be included');
		}
				
		// just set it op once
		if ( self::$setup == false ) 
		{	
			self::$setup = true;
			self::$path	 = $path;
				
			if ($enableClasses)
			{
				// Register the class map based autoloader.
				spl_autoload_register(array('BLoader', 'load'));
			}
					
			if ($enablePrefixes)
			{
				// Register namespaces to path namespaces
				## NOTE - Loading it here causes a loop WHY ????
				self::registerPrefix('B',			'app');
				self::registerPrefix('Bookzo', 		'bookzo');
				self::registerPrefix('BzHelper', 	'helpers');
		
				// Register the prefix autoloader.
				spl_autoload_register(array('BLoader', '_autoload'));
			}
		}
	}
	
	/**
	 * Load the file for a class.
	 *
	 * @param   string  $class  The class to be loaded.
	 *
	 * @return  boolean  True on success
	 *
	 * @since   11.1
	 */
	public static function load($class)
	{
		// Sanitize class name.
		$class = strtolower($class);

		// If the class already exists do nothing.
		if (class_exists($class, false))
		{
			return true;
		}

		// If the class is registered include the file.
		if (isset(self::$classes[$class]))
		{
			include_once self::$classes[$class];

			return true;
		}

		return false;
	}
	
	/**
	 * Autoload a class based on name.
	 *
	 * @param   string  $class  The class to be loaded.
	 *
	 * @return  boolean  True if the class was loaded, false otherwise.
	 *
	 * @since   11.3
	 */
	private static function _autoload($class)
	{
		foreach (self::$prefixes as $prefix => $namespace)
		{
			$chr = strlen($prefix) < strlen($class) ? $class[strlen($prefix)] : 0;

			if (strpos($class, $prefix) === 0 && ($chr === strtoupper($chr)))
			{
				return self::_load(substr($class, strlen($prefix)), $namespace);
			}
		}

		return false;
	}
	
	/**
	 * Load a class based on name and lookup array.
	 *
	 * @param   string  $class   The class to be loaded (wihtout prefix).
	 * @param   array   $lookup  The array of base paths to use for finding the class file.
	 *
	 * @return  boolean  True if the class was loaded, false otherwise.
	 *
	 * @since   12.1
	 */
	private static function _load($class, $namespace)
	{
		// Split the class name into parts separated by camelCase.
		$parts = preg_split('/(?<=[a-z0-9])(?=[A-Z])/x', $class);

		// If there is only one part we want to duplicate that part for generating the path.
		$parts = (count($parts) === 1) ? array($parts[0], $parts[0]) : $parts;

		foreach ($namespace as $base)
		{
			if ( $file = self::$path->path($base.':'.implode('/', array_map('strtolower', $parts)) . '.php') ) {
				return include $file;
			}
		}

		return false;
	}
}

/**
 * BzException identifies an Exception in the Bz class
 * @see Bz
 */
class BRuntimeException extends Exception {

	/**
	 * Converts the exception to a human readable string
	 *
	 * @return 	string The error message
	 * @since 	1.0.0
	 */
	public function __toString() 
	{
		return $this->getMessage();
	}

}