<?php
/**
 * @package     Bookzo.Platform
 * @subpackage  Form
 *
 * @copyright   Copyright (C) 2005 - 2014 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE
 */

defined('_BOOKZO') or die;

/**
 * Form Field class for the Joomla Platform.
 * Single check box field.
 * This is a boolean field with null for false and the specified option for true
 *
 * @package     Bookzo.Platform
 * @subpackage  Form
 * @link        http://www.w3.org/TR/html-markup/input.checkbox.html#input.checkbox
 * @see         BFormFieldCheckboxes
 * @since       11.1
 */
class BFormFieldBookableItem extends BFormFieldCheckbox
{
	/**
	 * The colors.
	 *
	 * @var    mixed
	 * @since  3.2
	 */
	protected $price;
	
	/**
	 * The colors.
	 *
	 * @var    mixed
	 * @since  3.2
	 */
	protected $costtype;
	
	/**
	 * The colors.
	 *
	 * @var    mixed
	 * @since  3.2
	 */
	protected $persons;
	
	/**
	 * The colors.
	 *
	 * @var    mixed
	 * @since  3.2
	 */
	protected $inlinelabel = false;
	
	/**
	 * Method to get certain otherwise inaccessible properties from the form field object.
	 *
	 * @param   string  $name  The property name for which to the the value.
	 *
	 * @return  mixed  The property value or null.
	 *
	 * @since   3.2
	 */
	public function __get($name)
	{
		switch ($name)
		{
			case 'price':
			case 'costtype':
				return $this->$name;
				
			case 'inlinelabel':
				return (bool) $this->$name;
				
			case 'persons':
				return (int) $this->$name;
		}

		return parent::__get($name);
	}

	/**
	 * Method to set certain otherwise inaccessible properties of the form field object.
	 *
	 * @param   string  $name   The property name for which to the the value.
	 * @param   mixed   $value  The value of the property.
	 *
	 * @return  void
	 *
	 * @since   3.2
	 */
	public function __set($name, $value)
	{
		switch ($name)
		{
			case 'price':
			case 'costtype' :
				$this->$name = (string) $value;
				break;
				
			case 'inlinelabel':
				$this->$name = (bool) $value;
				break;
				
			case 'persons':
				$this->$name = (int) $value;
				break;

			default:
				parent::__set($name, $value);
		}
	}
	
	/**
	 * Method to attach a JForm object to the field.
	 *
	 * @param   SimpleXMLElement  $element  The SimpleXMLElement object representing the <field /> tag for the form field object.
	 * @param   mixed             $value    The form field value to validate.
	 * @param   string            $group    The field name group control value. This acts as as an array container for the field.
	 *                                      For example if the field has name="foo" and the group value is set to "bar" then the
	 *                                      full field name would end up being "bar[foo]".
	 *
	 * @return  boolean  True on success.
	 *
	 * @see     BFormField::setup()
	 * @since   3.2
	 */
	public function setup(SimpleXMLElement $element, $value, $group = null)
	{
		$return = parent::setup($element, $value, $group);

		if ($return)
		{
			$inlinelabel = (bool) $this->element['inlinelabel'];

			$this->inlinelabel  = ($inlinelabel == 'true' || $inlinelabel == 'inlinelabel' || $inlinelabel == '1');
			$this->price  		= (string) $this->element['price'];
			$this->costtype		= (string) $this->element['costtype'];
			$this->persons		= (int) $this->element['persons'];
		}

		return $return;
	}
	
	/**
	 * Method to get the field input markup.
	 * The checked element sets the field to selected.
	 *
	 * @return  string  The field input markup.
	 *
	 * @since   11.1
	 */
	protected function getInput()
	{
		// Initialize some field attributes.
		$datastring = $this->getAttribute('datastring');
		$price		= $this->price;
		$type		= strtolower($this->costtype) != 'manual' ? 'box' : 'number';

		$checked   	= $this->checked || !empty($this->value) ? ' checked' : '';
		$disabled  	= $this->disabled 				? ' disabled' : '';
		$value 		= strlen($this->value) 			? (int) $this->value : 0;
		$required  	= $this->required 				? ' required aria-required="true"' : '';
		$autofocus 	= $this->autofocus 				? ' autofocus' : '';
		$datastring	= !empty($datastring)  			? ' '. $datastring .' ' : '';
		
		if ( $type == 'box' ) 
		{
			$value     	= 1;
			$class     	= !empty($this->class) 		? ' class="' . $this->class . '"' : '';
			$input		= '<input type="checkbox" name="' . $this->name . '" id="' . $this->id . '" value="'
						. htmlspecialchars($value, ENT_COMPAT, 'UTF-8') . '"' . $class . $checked . $disabled . $datastring
						. $required . $autofocus . ' />';
		}
		else {
			$class     	= !empty($this->class) 		? ' class="th-input-number ' . $this->class . '"' : 'class="th-input-number"';
			$input		= '<input type="number" name="' . $this->name . '" id="' . $this->id . '" value="'
						. (int) $value . '"' . $class . $checked . $disabled . $datastring
						. $required . $autofocus . ' min="0" />';
		}
			
					
		if ( (bool) $this->inlinelabel ) {
			
			$label = $this->element['label'] ? (string) $this->element['label'] : (string) $this->element['name'];
			$label = $this->translateLabel ? BText::_($label) : $label;
		
			$html = array();
			$html[] = '<div class="bz-grid">';
			$html[] = '<div class="bz-width-4-6">'.$input . ' '.  $label.'</div>';
			$html[] = '<div class="bz-width-2-6 bz-text-right">';
			if ( $this->costtype != 'PercentageOfRent' ) {
				$html[] = '<span class="currency">€</span> ';
			}
			$html[] = $price;
			if ( $this->costtype == 'PercentageOfRent' ) {
				$html[] = ' <span class="percentage">%</span> ';
			}
			if ( $this->costtype != 'PercentageOfRent' && $this->costtype != 'Manual' ) {
				$html[] = ' <span class="type">'.BText::_('BOOKABLE_ITEM_TYPE_'. $this->costtype).'</span> ';
			}
			$html[] = '</div>';
			$html[] = '</div>';
			
			return '<div class="bookable-item">' . implode($html) . '</div>';
		}
		else {
			return '<div class="bookable-item">' .$input . '</div>';
		}
	}
}
